unit KZDeskManagers;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, LMessages, ComCtrls, ExtCtrls, Dialogs,
  LCLIntf, LCLType, SrcEditorIntf, IDEMsgIntf;

type
  TKZParentType = (kzptParent, kzptParentWindow, kzptDock, kzptNone);

type

  TKZLMessageHandlerEvent = procedure(Sender: TObject; SenderCtrl: TControl; Msg: TLMessage) of object;

  { TKZLMessageHandler }

  TKZLMessageHandler = class(TPersistent)
  private

    Control: TControl;
    InheritedWndMethod: TWndMethod;

    FMessagesIgnore: TList;

    function ValidControl: Boolean;

  public

    OnBefforeMessage,
    OnAfterMessage: TKZLMessageHandlerEvent;

    procedure SelfWndMethod(var TheMessage: TLMessage);
    procedure DestroyMe(Sender: TObject);

    constructor Create(AControl: TControl);

    procedure AddMessageIgnore(LM_: Integer);
    procedure RemoveMessageIgnore(LM_: Integer);

  end;

  {
  TKZDeskManagerFormInfo = class
  public
    Form: TCustomForm;
    MessageHandler: TKZLMessageHandler;
  end;
  }

  { TKZDeskManager }

  TKZDeskManager = class
  private
    function getItems(AIndex: Integer): TCustomForm; // TKZDeskManagerFormInfo;
  protected

    FRemoveBorder: Boolean;

    FormParent: TWinControl;
    ParentType: TKZParentType;

    FItems: TList;

    function IsMessageShowing(LM: TLMessage): Boolean;

  public

    AutoShowHideForms: Boolean;

    property Items[AIndex: Integer]: TCustomForm { TKZDeskManagerFormInfo } read getItems;

    procedure FormParentResize(Sender: TObject);
    procedure FormParentVisibleChange(Sender: TObject);

    procedure Prepare(_Form: TCustomForm); virtual;
    procedure AddForm(_Form: TCustomForm); virtual;
    procedure RemoveForm(_Form: TCustomForm); virtual;
    procedure HideForms; virtual;
    procedure ShowForms; virtual;

    procedure BefforeMessage(Sender: TObject; SenderWindow: TControl; Msg: TLMessage); virtual;
    procedure AfterMessage(Sender: TObject; SenderWindow: TControl; Msg: TLMessage); virtual;

    constructor Create(_ParentControl: TWinControl; _ParentType: TKZParentType);
    destructor Destroy; override;

  end;

  { TKZCodeExplorerViewManager }

  TKZCodeExplorerViewManager = class(TKZDeskManager)
  public

    procedure BefforeMessage(Sender: TObject; SenderWindow: TControl; Msg: TLMessage); override;

  end;

  { TKZObjectInspectorManager }

  TKZObjectInspectorManager = class(TKZDeskManager)
  public

    procedure Prepare(_Form: TCustomForm); override;

    procedure BefforeMessage(Sender: TObject; SenderWindow: TControl; Msg: TLMessage); override;
    procedure AfterMessage(Sender: TObject; SenderWindow: TControl; Msg: TLMessage); override;

  end;

  { TKZMessagesViewManager }

  TKZMessagesViewManager = class(TKZDeskManager)
  public

    procedure Prepare(_Form: TCustomForm); override;

    procedure BefforeMessage(Sender: TObject; SenderWindow: TControl; Msg: TLMessage); override;
    procedure AfterMessage(Sender: TObject; SenderWindow: TControl; Msg: TLMessage); override;

  end;

  { TKZPackageManager }

  TKZPackageManager = class(TKZDeskManager)
  public

    procedure BefforeMessage(Sender: TObject; SenderWindow: TControl; Msg: TLMessage); override;

  end;

  { TKZSourceEditorManager }

  TKZSourceEditorManager = class(TKZDeskManager)
  public

    procedure AddForm(_Form: TCustomForm); override;
    procedure Prepare(_Form: TCustomForm); override;

    procedure ShowForms; override;
    procedure HideForms; override;

    procedure BefforeMessage(Sender: TObject; SenderWindow: TControl; Msg: TLMessage); override;
    procedure AfterMessage(Sender: TObject; SenderWindow: TControl; Msg: TLMessage); override;

  end;

implementation

{ TKZSourceEditorManager }

procedure TKZSourceEditorManager.AddForm(_Form: TCustomForm);
begin
  inherited AddForm(_Form);
end;

procedure TKZSourceEditorManager.Prepare(_Form: TCustomForm);
begin
  inherited Prepare(_Form);

end;

procedure TKZSourceEditorManager.ShowForms;
var
  i, a: Integer;
begin

  inherited ShowForms;

  // Veja a nota no 'HideForms;'. Aqui habilitamos novamente o SynEdit.
  for i := 0 to FItems.Count - 1 do
  begin
    with TSourceEditorWindowInterface(Items[i]) do
    begin
      for a := 0 to Count - 1 do
      begin
        Items[a].EditorControl.Enabled := True;
      end;

    end;
  end;

end;

procedure TKZSourceEditorManager.HideForms;
var
  i, a: Integer;
begin

  // Antes de esconder os forms editores, devemos desabilitar os SynEdits deles.
  // Isso é necessário pois após esconder/mostrar o form, a posição do cursor no
  // SynEdit se perde, e o editor fica como se alguém tivesse apertando a tecla
  // END no editor, indo para o final da linha. Desabilitar o editor durate o pro-
  // cesso impede que isso aconteça.
  for i := 0 to FItems.Count - 1 do
  begin
    for a := 0 to TSourceEditorWindowInterface(Items[i]).Count - 1 do
    begin
      TSourceEditorWindowInterface(Items[i]).Items[a].
        EditorControl.Enabled := False;
    end;
  end;

  inherited HideForms;

end;

procedure TKZSourceEditorManager.BefforeMessage(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);
begin
  inherited BefforeMessage(Sender, SenderWindow, Msg);

  // Se ele vai exibir, precisa exibir a tabsheet antes
  if IsMessageShowing(Msg) then
  begin

    if FormParent.Parent is TTabSheet then
    begin
      TTabSheet(FormParent.Parent).PageControl.ActivePage := TTabSheet(FormParent.Parent);
    end;

  end;

end;

procedure TKZSourceEditorManager.AfterMessage(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);
begin
  inherited AfterMessage(Sender, SenderWindow, Msg);

  if
    (SourceEditorManagerIntf <> Nil) and
    (SourceEditorManagerIntf.ActiveEditor <> Nil) and
    (SourceEditorManagerIntf.ActiveEditor.EditorControl <> Nil)
  then
  begin
    // Aqui damos foco no SynEdit. Isso é importante, pois como desabilitamos o
    // editor no 'HideForms;', quando o editor aparece, fica sem foco.
    with SourceEditorManagerIntf.ActiveEditor.EditorControl do
    begin
      if CanFocus then SetFocus;
    end;

  end;

end;

{ TKZPackageManager }

procedure TKZPackageManager.BefforeMessage(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);
begin
  inherited BefforeMessage(Sender, SenderWindow, Msg);
  {
  // Se ele vai exibir, precisa exibir a tabsheet antes
  if IsMessageShowing(Msg) then
  begin

    //if FormParent.Parent is TTabSheet then
    //begin
      //TTabSheet(FormParent.Parent).PageControl.ActivePage := TTabSheet(FormParent.Parent);
      //ShowMessage('PK!');
    //end;

  end;
  }
end;

{ TKZMessagesViewManager }

procedure TKZMessagesViewManager.Prepare(_Form: TCustomForm);
begin

  inherited Prepare(_Form);

  TPanel( FormParent ).BevelOuter := bvNone;

end;

procedure TKZMessagesViewManager.BefforeMessage(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);
begin
  inherited BefforeMessage(Sender, SenderWindow, Msg);

end;

procedure TKZMessagesViewManager.AfterMessage(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);
begin
  inherited AfterMessage(Sender, SenderWindow, Msg);

  //if IsMessageShowing(Msg) then FormParentResize(FormParent);

end;

{ TKZObjectInspectorManager }

procedure TKZObjectInspectorManager.Prepare(_Form: TCustomForm);
begin
  inherited Prepare(_Form);

  _Form.FindChildControl('StatusBar').Visible := False;

  // Teste
  //while _Form.ControlCount > 0 do
  //begin
  //  _Form.Controls[0].Parent := FormParent;
  //end;

end;

procedure TKZObjectInspectorManager.BefforeMessage(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);
begin
  inherited BefforeMessage(Sender, SenderWindow, Msg);

  if IsMessageShowing(Msg) then
  begin

    if FormParent is TTabSheet then
    with TTabSheet(FormParent) do
    begin
      PageControl.ActivePage := TTabSheet(FormParent);
    end;

  end;

  {
  if Msg.msg = LM_KEYDOWN {LM_SETEDITABLE} then
  begin
    LCLIntf.SetActiveWindow(TWinControl( SenderWindow ).Handle);
    //SenderWindow.Parent.SetFocus;
    //TWinControl( SenderWindow ).SetFocus;
    IDEMessagesWindow.AddMsg('bah', '', 0);
  end;
  }

end;

procedure TKZObjectInspectorManager.AfterMessage(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);
begin
  inherited AfterMessage(Sender, SenderWindow, Msg);

  //if Pos('Unk', GetMessageName(Msg.msg)) = 0 then
    //IDEMessagesWindow.AddMsg(FormatDateTime('HH:MM:ss', Now) + ': ' + GetMessageName(Msg.msg), '', 0);

end;

{ TKZCodeExplorerViewManager }

procedure TKZCodeExplorerViewManager.BefforeMessage(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);
begin
  inherited BefforeMessage(Sender, SenderWindow, Msg);

  if IsMessageShowing(Msg) then
  begin
    if FormParent is TTabSheet then
    with TTabSheet(FormParent) do
    begin
      PageControl.ShowTabs := True;
      PageControl.ActivePage := TTabSheet(FormParent);
    end;

  end;

end;

{ TKZLMessageHandler }

function TKZLMessageHandler.ValidControl: Boolean;
begin
  Result := (Assigned(Control)) and (Control <> Nil);
end;

procedure TKZLMessageHandler.SelfWndMethod(var TheMessage: TLMessage);
var
  i: Integer;
begin

  if (not ValidControl) then Exit;

  for i := 0 to FMessagesIgnore.Count - 1 do
  begin
    if Integer(FMessagesIgnore[i]) = TheMessage.msg then
    begin
      Exit;
    end;
  end;

  if Assigned(OnBefforeMessage) then
    OnBefforeMessage(Self, Control, TheMessage);

  if Assigned(InheritedWndMethod) then
    InheritedWndMethod(TheMessage);

  if Assigned(OnAfterMessage) then
    OnAfterMessage(Self, Control, TheMessage);

end;

procedure TKZLMessageHandler.DestroyMe(Sender: TObject);
begin
  Free;
end;

constructor TKZLMessageHandler.Create(AControl: TControl);
begin
  inherited Create;

  FMessagesIgnore := TList.Create;

  Control := AControl;

  InheritedWndMethod :=  Control.WindowProc;
  Control.WindowProc := @SelfWndMethod;

  //Control.AddHandlerOnBeforeDestruction(@DestroyMe);

end;

procedure TKZLMessageHandler.AddMessageIgnore(LM_: Integer);
begin
  FMessagesIgnore.Add(Pointer(LM_));
end;

procedure TKZLMessageHandler.RemoveMessageIgnore(LM_: Integer);
begin
  FMessagesIgnore.Remove(Pointer(LM_));
end;

{ TKZDeskManager }

function TKZDeskManager.getItems(AIndex: Integer): TCustomForm; // TKZDeskManagerFormInfo;
begin
  if FItems = Nil then
    Result := Nil
  else
    Result := TCustomForm(FItems[AIndex]);
end;

function TKZDeskManager.IsMessageShowing(LM: TLMessage): Boolean;
begin

  Result := (LM.msg = LM_SHOWWINDOW) and (TLMShowWindow(LM).Show);

end;

procedure TKZDeskManager.FormParentResize(Sender: TObject);
var
  i: Integer;
begin

  if FItems = Nil then Exit;

  for i := 0 to FItems.Count - 1 do
  begin
    //if (Items[i].Form <> Nil) and (Assigned(Items[i].Form)) then
    if Assigned( Items[i] ) then
    begin
      //with Items[i].Form do
      with Items[i] do
      begin
        Left := 0;
        Top := 0;
        Width  := FormParent.Width;
        Height := FormParent.Height;
      end;

    end;

  end;

end;

procedure TKZDeskManager.FormParentVisibleChange(Sender: TObject);
begin

  if (not AutoShowHideForms) or (FormParent = Nil) then Exit;

  if FormParent.Visible then
    ShowForms
  else
    HideForms;

end;

procedure TKZDeskManager.Prepare(_Form: TCustomForm);
begin

  AddForm(_Form);

end;

procedure TKZDeskManager.AddForm(_Form: TCustomForm);
var
  aRect: TRect;
  //item: TKZDeskManagerFormInfo;
begin

  with _Form do
  begin

    if Self.FRemoveBorder then
      BorderStyle := bsNone;

    case ParentType of
      kzptNone          : ;

      kzptDock          :
      begin
        aRect.Left := 0;
        aRect.Top := 0;
        aRect.Right := Width;
        aRect.Bottom := Height;
        Dock(FormParent, aRect);
      end;

      kzptParent        : Parent := FormParent;
      kzptParentWindow  : ParentWindow := FormParent.Handle;
    end;

    with TKZLMessageHandler.Create(_Form) do
    begin
      OnBefforeMessage := @BefforeMessage;
      OnAfterMessage := @AfterMessage;
    end;

  end;

  FItems.Add(_Form);

  {
  item := TKZDeskManagerFormInfo.Create;
  with item do
  begin
    Form := _Form;

    if Self.FRemoveBorder then
      Form.BorderStyle := bsNone;

    case ParentType of
      kzptNone          : ;

      kzptDock          :
      begin
        aRect.Left := 0;
        aRect.Top := 0;
        aRect.Right := Form.Width;
        aRect.Bottom := Form.Height;
        Form.Dock(FormParent, aRect);
      end;

      kzptParent        : Form.Parent := FormParent;
      kzptParentWindow  : Form.ParentWindow := FormParent.Handle;
    end;

    //MessageHandler := TKZLMessageHandler.Create(Form);
    with TKZLMessageHandler.Create(Form) do
    begin
      OnBefforeMessage := @BefforeMessage;
      OnAfterMessage := @AfterMessage;
    end;

  end;

  FItems.Add(item);
  }

end;

procedure TKZDeskManager.RemoveForm(_Form: TCustomForm);
var
  i: Integer;
begin

  if FItems = Nil then Exit;

  FItems.Remove(_Form);

  {
  for i := 0 to FItems.Count - 1 do
  begin

    if Items[i].Form = _Form then
    begin
      //Items[i].MessageHandler.Free;
      FItems.Remove(_Form);
      Break;
    end;

  end;
  }

end;

procedure TKZDeskManager.HideForms;
var
  i: Integer;
begin

  if FItems = Nil then Exit;

  for i := 0 to FItems.Count - 1 do
  begin

    if (Items[i] <> Nil) and (Assigned(Items[i])) then
    begin
      with Items[i] do
      begin

        Hide;

      end;
    end;

  end;

end;

procedure TKZDeskManager.ShowForms;
var
  i: Integer;
begin

  if FItems = Nil then Exit;

  for i := 0 to FItems.Count - 1 do
  begin

    if (Items[i] <> Nil) and (Assigned(Items[i])) then
    begin
      with Items[i] do
      begin

        Visible := True;

      end;

    end;

  end;

end;

procedure TKZDeskManager.BefforeMessage(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);
begin

  if Msg.msg = LM_DESTROY then
  begin
    RemoveForm(TCustomForm( SenderWindow ));
  end;

end;

procedure TKZDeskManager.AfterMessage(Sender: TObject; SenderWindow: TControl;
  Msg: TLMessage);
begin

  if IsMessageShowing(Msg) then
    FormParentResize(FormParent);

end;

constructor TKZDeskManager.Create(_ParentControl: TWinControl;
  _ParentType: TKZParentType);
begin

  FormParent       := _ParentControl;
  ParentType       := _ParentType;

  FItems := TList.Create;

  FormParent.AddHandlerOnResize(@FormParentResize);
  FormParent.AddHandlerOnVisibleChanged(@FormParentVisibleChange);

  FRemoveBorder := True;
  AutoShowHideForms := False;

end;

destructor TKZDeskManager.Destroy;
begin

  FreeAndNil(FItems);
  inherited Destroy;

end;

end.

