{
 *****************************************************************************
 *                                                                           *
 *  See the file COPYING.modifiedLGPL.txt, included in the lazarus           *
 *  directory, for details about the license/copyright.                      *
 *                                                                           *
 *  This program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *                                                                           *
 *****************************************************************************

  Author....: Raphael Zimermann
  Website...: www.raphaelz.com.br

  About:
    KZDesktop transforms the lazarus IDE to a layout similar to that of the
    Package ViewerPackage Viewer"Delphi Rad Studio" IDE.

}

unit kzdesktop;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LCLIntf, LCLType, LResources, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, TypInfo, Buttons, StdCtrls, GraphType, LMessages, MenuIntf, ComCtrls, Menus,
  IniFiles, IDEOptionsIntf, LazIDEIntf;

procedure Register;

  procedure SaveCFG(_Section, _Ident, _Value: String);
  function LoadCFG(_Section, _Ident: String; _Default: String = ''): String;

implementation

uses
  KZLazDesktop, uKZConfig;


function KzDesktopCFG: String;
begin
  Result := ExtractFilePath(Application.ExeName) + PathDelim + 'kzdesktop.cfg'; // LazApp
end;

procedure SaveCFG(_Section, _Ident, _Value: String);
var
  cfg: TIniFile;
begin

  cfg := TIniFile.Create(KzDesktopCFG);
  try
    cfg.WriteString(_Section, _Ident, _Value);
  finally
    FreeAndNil(cfg);
  end;

end;

function LoadCFG(_Section, _Ident: String; _Default: String = ''): String;
var
  cfg: TIniFile;
begin

  cfg := TIniFile.Create(KzDesktopCFG);
  try
    Result := cfg.ReadString(_Section, _Ident, _Default);
  finally
    FreeAndNil(cfg);
  end;

end;

procedure StartKzDeskTop(Sender: TObject);
begin

  if Sender <> nil then
  begin
    SaveCFG('Initialization', 'Started', 'True');

    ShowMessage('Done.' + #13#10 +
      'Remember: The KZDesktop is still under development' +
      'and has some bugs that are being corrected over time!' + #13#10 + #13#10 +
      'Restart Lazarus for the KZDesktop take effect.');
  end
  else
  begin
    KZLazDesktopInterface := TKZLazDesktopInterface.Create(Application);
    KZLazDesktopInterface.Execute;
  end;

end;

procedure StopKzDeskTop(Sender: TObject);
begin

  SaveCFG('Initialization', 'Started', 'False');
  ShowMessage('Done.' + #13#10 + 'Restart Lazarus to disable KZDesktop.')

  //if DeleteFile(KzDesktopCFG) then
  //  ShowMessage('Done.' + #13#10 + 'Restart Lazarus to disable KZDesktop.')
  //else
  //  ShowMessage('Problem to disable KZDesktop!');

end;

procedure AboutKzDeskTop(Sender: TObject);
begin

  ShowMessage(
  'KZDesktop (2011-2012)' + #13#10 +
  'This version is a alpha version. Ignore the bugs, please! ;)' + #13#10 +
  'By Raphael Zimermann - http://www.raphaelz.com.br/');

end;

procedure ConfigKzDeskTop(Sender: TObject);
begin

  LazarusIDE.DoOpenIDEOptions(TfmKZConfig);

  {fmKZConfig := TfmKZConfig.Create(Application);
  try
    fmKZConfig.Enabled := True;
    fmKZConfig.ShowModal;
  finally
    fmKZConfig.Release;
    fmKZConfig := Nil;
  end;}

end;

procedure StartMyTool_Teste(Sender: TObject);
var
  i, a: Integer;
  s: String;
begin

  ShowMessage('Screen.Forms');

  for i := 0 to Screen.FormCount - 1 do
  begin
    s := Screen.Forms[i].Name + ': ' + Screen.Forms[i].ClassName;

    for a := 0 to Screen.Forms[i].ComponentCount - 1 do
    begin
      s := s + #13#10 + '...' + Screen.Forms[i].Components[a].Name + ': ' +
        Screen.Forms[i].Components[a].ClassName;
    end;

    ShowMessage(s);

  end;

  ShowMessage('Application.Components');

  for i := 0 to Application.ComponentCount - 1 do
  begin
    s := Application.Components[i].Name + ': ' + Application.Components[i].ClassName;

    for a := 0 to Application.Components[i].ComponentCount - 1 do
    begin
      s := s + #13#10 + '...' + Application.Components[i].Components[a].Name + ': ' +
        Application.Components[i].Components[a].ClassName;
    end;

    ShowMessage(s);
  end;

  //Screen.FindForm('fmPlayList').Parent := Screen.FindForm('fmPrincipal');

  //Screen.FindForm('fmPlayList').ParentWindow := Screen.FindForm('fmPrincipal').Handle;

end;

procedure Register;
var
  mm: TIDEMenuSection;
begin

  mm := itmSecondaryTools;

  RegisterIDEMenuCommand(mm, 'mmKZDesktopSep', '-', nil, nil);

  if (not FileExists(KzDesktopCFG)) or (LoadCFG('Initialization', 'Started', 'False') = 'False') then
  begin
    RegisterIDEMenuCommand(mm, 'mmKZDesktopStart', 'KZ Desktop - Start', nil, @StartKzDeskTop);
  end
  else
  begin
    RegisterIDEMenuCommand(mm, 'mmKZDesktopStop', 'KZ Desktop - Stop', nil, @StopKzDeskTop);
    RegisterIDEOptionsEditor(GroupEnvironment, TfmKZConfig, 1000);
    StartKzDeskTop(Nil);
  end;

  RegisterIDEMenuCommand(mm, 'mmConfigKzDeskTop', 'KZ Desktop - Options', nil, @ConfigKzDeskTop);
  RegisterIDEMenuCommand(mm, 'mmKZDesktopAbout', 'KZ Desktop - About', nil, @AboutKzDeskTop);

end;

end.






///
