{
 *****************************************************************************
 *                                                                           *
 *  See the file COPYING.modifiedLGPL.txt, included in the lazarus           *
 *  directory, for details about the copyright/license.                      *
 *                                                                           *
 *  This program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     *
 *                                                                           *
 *****************************************************************************

  Author....: Raphael Zimermann
  Website...: www.raphaelz.com.br

  About:
    KZDesktop transforms the lazarus IDE to a layout similar to that of the
    "Delphi Rad Studio" IDE.

  My public SVN: svn://svn.code.sf.net/p/kzdesktop/code/trunk





  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  Notes: [PT-BR]

  Abaixo imformações interessantes, sobre o comportamento do processo
  nos variados sistemas operacionais:

  - Quando colocamos uma janela ou componente dentro de outra (por meio do
    ParentWindow ou SetParent), a janela ou componente fica impossibilitada
    de receber foco. Esse bug é até mesmo reconhecido pela microsoft. Para
    a janela receber foco, seu parent deve receber o foco. Porisso ao chamar
    as janelas que tem um parent, é necessário dar foco neste parent!

  - No windows, quando fazemos o tratamento da posição do form no designer, as linhas
    e posição do TDesigner se perdem; Para resolver este problema, basta executar o
    "UpdateControlState" no form. PORÉM, o UpdateControlState é um protected, e não
    pode ser chamado diretamente. Para ser chamado indiretamente, deve-se usar o
    comando "with...do". Não sei porque isso ocorre, talvez seja erro do compilador,
    mas no fim das contas funciona.

  - Problema com os LMessages? Note que algumas constantes de mensagens são a soma
    de uma mensagem com outra ou com um valor. Nestes casos, usar um 'in' não funciona;
    tornase necessário usar o '=' no lugar, para verificar se a mensagem em questão
    é a mensagem LM_ABC...

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------



}

unit KZLazDesktop;

{$mode objfpc}{$H+}

//{$DEFINE MODO_TESTE}
//{$DEFINE TESTE2}
//{$DEFINE TESTE_MSGS}

interface

uses
  Classes, SysUtils, Math,
  Forms, WSForms, Controls, Graphics, Dialogs, ExtCtrls,
  ComCtrls, LMessages,

  MenuIntf, SrcEditorIntf, ObjectInspector, IDEMsgIntf, ProjectIntf,
  LazIDEIntf, IDEWindowIntf, ComponentEditors, FormEditingIntf,
  ComponentReg, IDEOptionsIntf,

  KZDeskManagers, process,
  LCLIntf,
  LCLType,
  LCLClasses,
  intfgraphics, StdCtrls, Buttons, EditBtn, Menus,
  dateutils, types, typinfo;

const
  PreviewFactor: Single = 7;


const
  KZ_USE_KZTOOLS        = False;
  KZ_DOCK_SOURCE_EDITOR = True;
  KZ_DOCK_FORM_DESIGN   = True;
  KZ_DOCK_PACKAGE       = True;
  KZ_REMOVE_FORM_BORDER = True;

  KZ_PACKAGE_PARENT_TYPE          : TKZParentType = kzptParent;        // Default: kzptParent
  KZ_OBJECT_INSPECTOR_PARENT_TYPE : TKZParentType = kzptParent;        // Default: kzptParent
  KZ_SOURCE_EDITOR_PARENT_TYPE    : TKZParentType = kzptParentWindow;  // Default: kzptParentWindow
  KZ_FORM_DESIGNER_PARENT_TYPE    : TKZParentType = kzptParentWindow;  // Default: kzptParentWindow

type


  { TDesignPanel }

  TDesignPanel = class(TPanel)
  protected

    FLoadingForm: Boolean;

    FFakeWindow: TPanel;
    FFakeWindow_MouseDown: Boolean;
    FFakeWindow_MousePos: TPoint;
    FFakeWindowCaption: TPanel;

    //FDefaultClientOrigin: TPoint;
    //FDefaultClientOriginPrepared: Boolean;

    procedure InitializeWnd; override;

    procedure SetupScreen;
    //procedure PrepareFormsOrigin;

    function FormIsValid(Form: TCustomForm): Boolean;
    procedure HideWindowDesign(Form: TCustomForm);
    function IsFormDesign(fm: TCustomForm): boolean;
    function LoadedFormIsValid: Boolean;
    function FormZeroOrigin(Form: TCustomForm): TPoint;

    procedure AddForm(Form: TCustomForm);
    procedure UnLoadForm(Form: TCustomForm);

  public

    FExecuteBefforeMessage: Boolean;

    LoadedForm,
    FLastForm: TCustomForm;
    OnFormLoad,
    OnLoadedFormChangeBounds: TNotifyEvent;

    procedure Screen_FormAdded(Sender: TObject; Form: TCustomForm);

    procedure Form_BefforeMessage(Sender: TObject; SenderForm: TControl; Msg: TLMessage);
    procedure Form_AfterMessage(Sender: TObject; SenderForm: TControl; Msg: TLMessage);

    procedure FFakeWindowMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FFakeWindowMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FFakeWindowMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FFakeWindowResize(Sender: TObject);
    procedure FFakeWindowCaptionPaint(Sender: TObject);

  published

    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;

    procedure UnLoadLoadedForm;
    procedure LoadLastForm;
    procedure UpdateScreenForms;

  end;

  { TFindComponentButton }

  TFindComponentButton = class(TSpeedButton)
  private
    ComponentButton: TSpeedButton;
    ComponentPage: TTabSheet;
    findEdt: TCustomEdit;
    FClassC: String;

    Comp: TRegisteredComponent;

  public


    procedure Click; override;
    procedure Prepare(_ComponentButton: TSpeedButton; _ComponentPage: TTabSheet;
      _findEdt: TCustomEdit; _Class: String);

    procedure Prepare(_Comp: TRegisteredComponent);

  end;

  { TTabComponentSearch }

  TTabComponentSearch = class(TPanel)
  public
    MyLabel: TLabel;
    MyPanel: TPanel;
    procedure DestroyButtons;
    function NewButton: TFindComponentButton;

    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;

  end;

  { TKZLazDesktopInterface }

  TKZLazDesktopInterface = class(TForm)
    Bevel1: TBevel;
    Button1: TButton;
    Button2: TButton;
    edFindComponent: TEditButton;
    imgDesignAreaView: TPaintBox;
    imgWindowScreenPreview: TImage;
    Label1: TLabel;
    MenuItem1: TMenuItem;
    PackageList: TListBox;
    Panel1: TPanel;
    pnKZTools: TPanel;
    pnDesignAreaView: TPanel;
    pnCaptionWindowScreenDesignAreaView: TPanel;
    pnScreenMove: TPanel;
    pnCodeEditor: TPanel;
    pnFindComponent: TPanel;
    pnCaptionWindowScreenPreview: TPanel;
    pcInspectors: TPageControl;
    pnWindowScreenPreview: TPanel;
    pnScreenPreview: TPanel;
    pnComponents: TPanel;
    pnDesign: TPanel;
    pcCorpo: TPageControl;
    pnCorpo: TPanel;
    pnPackage: TPanel;
    pnMessages: TPanel;
    pnMenu: TPanel;
    pnWindowScreenDesignAreaView: TPanel;
    ppmAndroid: TPopupMenu;
    MyProcess: TProcess;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    SpeedButton5: TSpeedButton;
    Splitter1: TSplitter;
    spObjectInspector: TSplitter;
    Splitter3: TSplitter;
    tsCodeExplorer: TTabSheet;
    tsObjectInspector: TTabSheet;
    tsCodeEditor: TTabSheet;
    tsFormEditor: TTabSheet;
    tsPackage: TTabSheet;
    procedure Button1Click(Sender: TObject);
    procedure edFindComponentButtonClick(Sender: TObject);
    procedure edFindComponentChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure imgDesignAreaViewPaint(Sender: TObject);
    procedure imgWindowScreenPreviewMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: integer);
    procedure imgWindowScreenPreviewMouseMove(Sender: TObject;
      Shift: TShiftState; X, Y: integer);
    procedure imgWindowScreenPreviewMouseUp(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: integer);
    procedure MenuItem1Click(Sender: TObject);
    procedure PackageListClick(Sender: TObject);
    procedure pcCorpoChange(Sender: TObject);
    procedure pnCodeEditorResize(Sender: TObject);
    procedure pnDesignAreaViewMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pnDesignAreaViewMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure pnDesignAreaViewMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pnPackageResize(Sender: TObject);
    procedure pnScreenMoveResize(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton5Click(Sender: TObject);
    procedure tsCodeEditorEnter(Sender: TObject);
    procedure tsFormEditorShow(Sender: TObject);
    procedure tsObjectInspectorResize(Sender: TObject);
    procedure tsObjectInspectorShow(Sender: TObject);
    procedure TESTE(Sender: TObject);
  private
    { private declarations }

    FPreviewMoving: boolean;
    FPreviewMovingPoint: TPoint;

    FDesignAreaViewMoving: boolean;
    FDesignAreaViewMovingPoint: TPoint;

    FDesignPanel: TDesignPanel;

    procedure MainIDEFormShow(TheOwner: TObject);

    function PixelPerc(vl, perc: single): integer;

  private
    { private 'Is' }

    function IsSourceEditor(fm: TCustomForm): boolean;
    function IsObjectInspector(fm: TCustomForm): boolean;
    function IsMessages(fm: TCustomForm): boolean;
    function IsFormDesign(fm: TCustomForm): boolean;
    function IsPackage(fm: TCustomForm): boolean;
    function IsCodeExplorerView(fm: TCustomForm): boolean;

  private
    { private objects }

    TabComponentSearch: TTabComponentSearch;
    PackageListHandles: TList;

    function MainIDE: TLazIDEInterface;
    function MainIDEForm: TCustomForm;
    function ObjectInspectorDlg: TObjectInspectorDlg;
    function MessagesView: TIDEMessagesWindowInterface;
    function CodeExplorerView: TCustomForm;

    function pnlSpeedButtons: TPanel;
    function ComponentPageControl: TPageControl;

  private
    { Private Managers }

    CodeExplorerViewManager: TKZCodeExplorerViewManager;
    ObjectInspectorManager: TKZObjectInspectorManager;
    MessagesViewManager: TKZMessagesViewManager;
    PackageManager: TKZPackageManager;
    SourceEditorManager: TKZSourceEditorManager;

  public
    { public setups }

    FEnterInEditor: boolean;

    procedure SetupPreviewFormDesign(_Form: TCustomForm);
    procedure SetupFormDesignAreaView(_Form: TCustomForm);
    procedure UpdatePackageList(AddForm: TCustomForm = Nil);

    //procedure SetupObjectInspector;
    //procedure SetupMessages;
    //procedure SetupCodeExplorerView;
    procedure SetupPackage(pkg: TCustomForm);
    procedure SetupSourceEditor(_Editor: TCustomForm);

    procedure OnIDERestoreWindows(Sender: TObject);
    procedure MainIDEFormResize(Sender: TObject);
    function OnProjectClose(Sender: TObject; AProject: TLazProject): TModalResult;
    procedure BefforeMessageWindow(Sender: TObject; SenderWindow: TControl; Msg: TLMessage);
    procedure AfterMessageWindow(Sender: TObject; SenderWindow: TControl; Msg: TLMessage);
    procedure AfterMessageEditor(Sender: TObject; SenderWindow: TControl; Msg: TLMessage);

    procedure DesignPanel_OnLoadForm(Sender: TObject);
    procedure DesignPanel_OnLoadedFormChangeBounds(Sender: TObject);
    procedure Screen_FormAdded(Sender: TObject; Form: TCustomForm);

    procedure SetupScreen;
    procedure SetupDefaultBars;

  public
    { public declarations }

    procedure UpdateFromOptions;
    procedure SaveLog(sLog: string);

    procedure Execute;

  end;

  procedure AddMessageDebug(_Msg: String);


var
  KZLazDesktopInterface: TKZLazDesktopInterface;

implementation

uses
  kzdesktop, uKZConfig;

procedure AddMessageDebug(_Msg: String);
begin
  {$IFDEF TESTE_MSGS}
  if IDEMessagesWindow <> Nil then
    IDEMessagesWindow.AddMsg(FormatDateTime('HH:mm:ss', Now) + ' ' + _Msg, '', 0);
  {$ENDIF}
end;


{ TDesignPanel }

procedure TDesignPanel.SetupScreen;
begin

  Screen.AddHandlerFormAdded(@Screen_FormAdded);

end;

function TDesignPanel.FormIsValid(Form: TCustomForm): Boolean;
begin
  Result := (Assigned(Form)) and (Form <> Nil);
end;

{
procedure TDesignPanel.PrepareFormsOrigin;
var
  fm: TForm;
begin

  if FDefaultClientOriginPrepared then Exit;

  FDefaultClientOriginPrepared := True;

  // Guarda o ponto de origem. Isso é importante para todos os forms
  // que forem carregados, pois não há como pegar o ClientOrigin corretamente
  // enquanto ele está processando as messages. Para que serve esse ponto?
  // Para saber a posição do form sem a borda... ;)
  fm := TForm.Create(Self);
  try
    with fm do
    begin
      ParentWindow := Self.Handle;
      Visible := True; // Importante!

      FDefaultClientOrigin.x := ClientOrigin.x - Left;
      FDefaultClientOrigin.y := ClientOrigin.y - Top;

    end;

  finally
    fm.Release;
    fm := Nil;
  end;

end;
}

procedure TDesignPanel.HideWindowDesign(Form: TCustomForm);
begin

  if (Form = Nil) or (not Assigned(Form)) then Exit;

  ShowWindow(Form.Handle, SW_HIDE);

end;


function TDesignPanel.IsFormDesign(fm: TCustomForm): boolean;
begin

  Result := ((fm <> Nil) and (Assigned(fm))) and ((csDesignInstance in fm.ComponentState) or
    (fm.ClassName = 'TNonControlDesignerForm') or (fm.ClassName =
    'TFrameDesignerForm'));

end;

function TDesignPanel.LoadedFormIsValid: Boolean;
begin
  Result := FormIsValid(LoadedForm);
end;

function TDesignPanel.FormZeroOrigin(Form: TCustomForm): TPoint;
var
  w: TWinControl;
  rc1, rc2: TRect;
  i: Integer;
begin

  if not KZ_REMOVE_FORM_BORDER then
  begin
    Result.x := 0;
    Result.y := 0;
    Exit;
  end;

  if FormIsValid(Form) then
  begin

    try
      w := TWinControl.Create(Self); // Tem que ser em Self se não dá pau!
      w.Left := 0;
      w.Top := 0;
      w.Parent := Form;

      GetWindowRect(w.Handle, rc1);
      GetWindowRect(Form.Handle, rc2);

      // Uso o 'Max' para evitar um rect negativo (acontece no ubuntu)
      Result.x := Max(rc1.Left - rc2.Left, 0);
      Result.y := Max(rc1.Top  - rc2.Top,  0);

      // Importante: Se tiver um menu no form, precisa colocar essa diferença!
      for i := 0 to Form.ComponentCount - 1 do
      begin
        if Form.Components[i] is TMainMenu then
        begin
          Result.y := Result.y - GetSystemMetrics(SM_CYMENU);
        end;
      end;

    finally
      FreeAndNil(w);
    end;

  end
  else
  begin
    Result.x := 0;
    Result.y := 0;
  end;

end;

procedure TDesignPanel.Form_AfterMessage(Sender: TObject; SenderForm: TControl;
  Msg: TLMessage);
var
  pRect: TRect;

  procedure DockFakeWindow(Form: TCustomForm);
  begin

    with FFakeWindow do
    begin

      Visible := ((Form <> Nil) and (Assigned(Form)));

      if Visible then
      begin

        Left    := ((Form.Left) - (FFakeWindowCaption.Left)) + FormZeroOrigin(Form).x;
        Top     := ((Form.Top) - (FFakeWindowCaption.Top + FFakeWindowCaption.Height)) + FormZeroOrigin(Form).y;

        Width   := Form.Width + (FFakeWindowCaption.Left * 2);
        Height  := Form.Height + (FFakeWindowCaption.Top + FFakeWindowCaption.Height) + FFakeWindowCaption.Top;

      end;

      FFakeWindowResize(FFakeWindow); // Força o resize, que não acontece se
                                      // tudo já estiver na mesma posição...

    end;

  end;

  procedure FocusFakeWindow;
  begin

    if FFakeWindow.Visible then
    begin
      Self.Left := (-FFakeWindow.Left) + 5;
      Self.Top  := (-FFakeWindow.Top)  + 5;
    end
    else
    begin
      Self.Left := (-LoadedForm.Left) + 5;
      Self.Top  := (-LoadedForm.Top)  + 5;
    end;

  end;

  procedure _UpdateFormBorder(Form: TCustomForm);
  var
    rgn: HRGN;
  begin

    if not KZ_REMOVE_FORM_BORDER then Exit;

    with Form, FormZeroOrigin(Form) do
    begin

      // Tira a borda do form. Esta é a melhor maneira de se fazer isso,
      // pois não prejudica as posições e tamanhos. ;)
      rgn := CreateRectRgn(
        x,
        y,
        x + Width,
        y + Height);

      SetWindowRgn(Handle, rgn, True);

    end;

  end;

  procedure _UpdateFormDesigner(Form: TCustomForm);
  begin
    // Obs: Tem que ser feito com o "do", conforme
    // observação no cabeçário da fonte...
    with TCustomForm( Form ) do UpdateControlState;
  end;

begin


  // Se a tela foi exibida...
  if (Msg.msg = LM_SHOWWINDOW) then
  begin

    // Se realmente está mostrando...
    if TLMShowWindow(Msg).Show then
    begin

      // Se o formulário é o formulário carregado...
      if TCustomForm(SenderForm) = LoadedForm then
      begin

        // Ajusta o FakeWindow
        DockFakeWindow(LoadedForm);

        // Atualiza a borda do formulário
        _UpdateFormBorder(LoadedForm);

        // Foca o designer no formulário
        FocusFakeWindow;

        // Importante: Seta o foco no Self (Pelo sistema! 'SetFocus' não funfa!)
        LCLIntf.SetFocus(Self.Handle); // Isso é importante pois as janelas modal não trabalham
                                       // o foco; Para que elas estejam em foco, o seu Parent
                                       // deve receber o foco.

        // Atualiza o desenho do Self, pois às vezes dá umas 'zicas'...
        //Self.Update; Self.Parent.Update;

      end;

    end;

  end;


  // Se o formulário foi movido...
  if (Msg.msg = LM_MOVE) then
  begin

    // Se o formulário é o formulário carregado...
    if TCustomForm(SenderForm) = LoadedForm then
    begin

      //AddMessageDebug(LoadedForm.Name + format(' after move (a), x=%d ; y=%d',
      //  [LoadedForm.left, LoadedForm.top]));


      // Ajusta o FakeWindow
      DockFakeWindow(LoadedForm);

      // Atualiza a borda do formulário
      _UpdateFormBorder(LoadedForm);

      // Foca o designer no formulário
      FocusFakeWindow;

      // Atualiza o Designer (para não dar zica na posição do TDesigner)
      _UpdateFormDesigner(LoadedForm);

      // Executa o evento de ao alterar tamanho/posição
      if Assigned(OnLoadedFormChangeBounds) then
        OnLoadedFormChangeBounds(Self);

      //AddMessageDebug(LoadedForm.Name + format(' after move (b), x=%d ; y=%d',
      //    [LoadedForm.left, LoadedForm.top]));

    end;

  end;


  // Se o formulário foi redimensionado...
  if (Msg.msg = LM_SIZE) then
  begin

    // Se o formulário é o formulário carregado...
    if TCustomForm(SenderForm) = LoadedForm then
    begin

      //AddMessageDebug(LoadedForm.Name + format(' after size (a), x=%d ; y=%d',
      //  [LoadedForm.left, LoadedForm.top]));

      // Ajusta o FakeWindow
      DockFakeWindow(LoadedForm);

      // Atualiza a borda do formulário
      _UpdateFormBorder(LoadedForm);

      // Foca o designer no formulário
      FocusFakeWindow;

      // Executa o evento de ao alterar tamanho/posição
      if Assigned(OnLoadedFormChangeBounds) then
        OnLoadedFormChangeBounds(Self);

      //AddMessageDebug(LoadedForm.Name + format(' after size (b), x=%d ; y=%d',
      //    [LoadedForm.left, LoadedForm.top]));

    end;

  end;


end;


procedure TDesignPanel.Form_BefforeMessage(Sender: TObject;
  SenderForm: TControl; Msg: TLMessage);
var
  pRect: TRect;

  procedure AdjustSelfPosition(_Form: TCustomForm);
  begin

    EXIT; // Isso não é mais necessário...

    // Posiciona o painel no espaço real de trabalho (Desktop),
    // para que quando a janela seja movida, a conversão de posições
    // seja perfeita...

    if _Form.ParentWindow <> Self.Handle then
    begin
      Self.Left := 0;
      Self.Top  := 0;

    end;

    GetWindowRect(Self.Handle, pRect);

    Self.Left := -(pRect.Left - Self.Left);
    Self.Top  := -(pRect.Top - Self.Top);


  end;

begin


  if not FExecuteBefforeMessage then Exit;

  if Msg.msg = LM_ACTIVATE then
  begin
    AddMessageDebug('CM_[BAH]');
  end;

  // Se está exibindo a tela...
  if (Msg.msg = LM_SHOWWINDOW) then
  begin

    AddMessageDebug('LM_SHOWWINDOW');

    // Se realmente está mostrando...
    if TLMShowWindow(Msg).Show then
    begin

      // Se havia um formulário carregado, descarrega ele
      if LoadedFormIsValid then
      begin
        UnLoadLoadedForm;
      end;

      // Define o formulário carregado
      LoadedForm := TCustomForm(SenderForm);

      // Se é a primeira vez que ele está sendo carregado...
      if (LoadedForm.ParentWindow <> Self.Handle) then
      begin

        // Posiciona o painel no espaço real de trabalho (Desktop),
        // para que quando a janela seja movida, a conversão de posições
        // seja perfeita...
        //AdjustSelfPosition(LoadedForm); OBS: Isso não é mais necessário...


        //AddMessageDebug(LoadedForm.Name + format(' loaded (a), x=%d ; y=%d',
        //    [LoadedForm.left, LoadedForm.top]));

        try
          FExecuteBefforeMessage := False; // Quando seta-se o parent no linux, ele
                                           // ele executa o after message de novo,
                                           // causando um erro de invalid point operation,
                                           // pois a memória ainda não está pronta!

          LoadedForm.BeginUpdateBounds; // O 'AdjustSelfPosition' não é mais necessário
                                        // pois iniciar o update de bounds já impede
                                        // que dê zica na posição do form.


          pRect.Left := LoadedForm.Left;
          pRect.Top := LoadedForm.Top;
          pRect.Right := LoadedForm.Left + LoadedForm.Width;
          pRect.Bottom := LoadedForm.Top + LoadedForm.Height;

          // Prende a tela no painel
          case KZ_FORM_DESIGNER_PARENT_TYPE of
            kzptParentWindow  : LoadedForm.ParentWindow := Self.Handle; // Padrão
            kzptParent        : LoadedForm.Parent := Self;
            kzptDock          : LoadedForm.Dock(Self, pRect);
            kzptNone          : ;
          end;



        finally
          LoadedForm.EndUpdateBounds;
          FExecuteBefforeMessage := True;
        end;

        {
        // Guarda o ponto de origem. Isso é importante para todos os forms
        // que forem carregados, pois não há como pegar o ClientOrigin corretamente
        // enquanto ele está processando as messages. Para que serve esse ponto?
        // Para saber a posição do form sem a borda... ;)
        PrepareFormsOrigin;
        }

        //AddMessageDebug(LoadedForm.Name + format(' loaded (b), x=%d ; y=%d',
        //    [LoadedForm.left, LoadedForm.top]));

      end;

      // Tendo lido (primeira vez ou não), executa o OnLoad
      if Assigned(OnFormLoad) then OnFormLoad(Self);

      // Executa o evento de Ao Alterar tamanho/posição (ora pois, isso
      // deve ser excutado na primeira vez também ;) )
      if Assigned(OnLoadedFormChangeBounds) then
        OnLoadedFormChangeBounds(Self);

      // Guarda como o último form carregado
      FLastForm := LoadedForm;

    end;


  end;

  // Se está prestes a mover a tela...
  if (Msg.msg in [LM_MOVE]) then
  begin

    // Se o formulário é o formulário carregado...
    if TCustomForm(SenderForm) = LoadedForm then
    begin

      //AddMessageDebug(LoadedForm.Name + format(' beffore move (a), x=%d ; y=%d',
      //  [LoadedForm.left, LoadedForm.top]));

      // não faz nada;

      //AddMessageDebug(LoadedForm.Name + format(' beffore move (b), x=%d ; y=%d',
      //  [LoadedForm.left, LoadedForm.top]));

    end;

  end;


  // Se está prestes a redimensionar a tela...
  if (Msg.msg in [LM_SIZE]) then
  begin

    // Se o formulário é o formulário carregado...
    if TCustomForm(SenderForm) = LoadedForm then
    begin

      //AddMessageDebug(LoadedForm.Name + format(' beffore size (a), x=%d ; y=%d',
      //  [LoadedForm.left, LoadedForm.top]));

      // não faz nada;

      //AddMessageDebug(LoadedForm.Name + format(' beffore size (b), x=%d ; y=%d',
      //  [LoadedForm.left, LoadedForm.top]));

    end;

  end;


end;

procedure TDesignPanel.FFakeWindowMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

  if
    (x >= FFakeWindow.Width - 3) or
    (y >= FFakeWindow.Height - 3)
  then
  begin
    FFakeWindow_MouseDown := True;
    GetCursorPos(FFakeWindow_MousePos);

  end;

end;

procedure TDesignPanel.FFakeWindowMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
const
  minWidth = 100;
  minHeight = 50;
var
  newPos: TPoint;
  frmPoint : TPoint;
  w, h: Integer;
begin

  with TWinControl(Sender) do
  begin

    if
      (x >= FFakeWindow.Width - 3) or
      (y >= FFakeWindow.Height - 3)
    then
      Cursor := crSizeNWSE
    else
      Cursor := crDefault;

    if FFakeWindow_MouseDown then
    begin

      GetCursorPos(newPos);

      frmPoint := ScreenToClient(Mouse.CursorPos);

      if frmPoint.X > minWidth then
        Width := frmPoint.X;

      if frmPoint.Y > minHeight then
        Height := frmPoint.Y;

      if LoadedFormIsValid then
      begin

        w := FFakeWindowCaption.Width; // Width  - 3 - 3;
        h := Height - ((FFakeWindowCaption.Top * 2) + FFakeWindowCaption.Height);

        try
          //LoadedForm.BeginUpdateBounds; Não faça!

          LoadedForm.Width  := w;
          LoadedForm.Height := h;

        finally
          //LoadedForm.EndUpdateBounds; Não faça!
        end;

        if Assigned(OnLoadedFormChangeBounds) then
          OnLoadedFormChangeBounds(Self);

      end;

    end;

  end;

end;

procedure TDesignPanel.FFakeWindowMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

  if FFakeWindow_MouseDown then
  begin
    FFakeWindow_MouseDown := False;
  end;

end;

procedure TDesignPanel.FFakeWindowResize(Sender: TObject);
var
  fk_rgn, fm_rgn, Region, Region2 : HRGN;
  fk_rc, fm_rc: TRect;
begin

  // Faz um furo quadrado no WindowFake, para quando o form ficar atrás dele
  // por causade um bug. Isso não ocorre no windows, mas é bom padronizar
  // e executar em todos os sistemas.

  fk_rc.Left   := 0;
  fk_rc.Top    := 0;
  fk_rc.Right  := FFakeWindow.Width;
  fk_rc.Bottom := FFakeWindow.Height;

  with fm_rc do
  begin
    Left   := FFakeWindowCaption.Left;
    Top    := FFakeWindowCaption.Top + FFakeWindowCaption.Height;
    Right  := Left + FFakeWindowCaption.Width;
    Bottom := FFakeWindow.Height - (FFakeWindowCaption.Top);
  end;

  fk_rgn := CreateRectRgn(fk_rc.Left, fk_rc.Top, fk_rc.Right, fk_rc.Bottom);
  fm_rgn := CreateRectRgn(fm_rc.Left, fm_rc.Top, fm_rc.Right, fm_rc.Bottom);

  CombineRgn(fk_rgn, fk_rgn, fm_rgn, RGN_DIFF);
  SetWindowRgn(FFakeWindow.Handle, fk_rgn, True);

end;

procedure TDesignPanel.FFakeWindowCaptionPaint(Sender: TObject);
begin

  if not LoadedFormIsValid then Exit;

  FFakeWindowCaption.Canvas.Font.Color := clHighlightText;
  FFakeWindowCaption.Canvas.TextOut(3, 0, LoadedForm.Caption);

end;

procedure TDesignPanel.InitializeWnd;
var
  fm: TForm;
begin
  inherited InitializeWnd;

  with FFakeWindow do
  begin
    // Essas coisas são feitas aqui porque no linux
    // no OnCreate o handle ainda não está pronto!
    //ParentWindow := Self.Handle;
    Parent := Self;
    ParentColor := False;
    Color := clForm;
  end;

end;

procedure TDesignPanel.AddForm(Form: TCustomForm);
begin

  HideWindowDesign(Form);

  if
    (Form.ParentWindow = Self.Handle) or
    (Form.Parent = Self)
  then
    Exit;

  with TKZLMessageHandler.Create(Form) do
  begin

    OnBefforeMessage := @Self.Form_BefforeMessage;
    OnAfterMessage   := @Self.Form_AfterMessage;

  end;

end;

procedure TDesignPanel.UnLoadForm(Form: TCustomForm);
begin

  if Form = nil then Exit;

  HideWindowDesign(Form);

  FFakeWindow.Visible := False;

end;

procedure TDesignPanel.Screen_FormAdded(Sender: TObject; Form: TCustomForm);
begin

  if not KZ_DOCK_FORM_DESIGN then Exit;

  // Aqui é a interceptação principal de quando um
  // form é adicionado!
  if IsFormDesign(Form) then
  begin
    AddForm(Form);
  end;

end;

constructor TDesignPanel.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);

  BevelInner := bvNone;
  BevelOuter := bvNone;

  FLoadingForm    := False;

  FExecuteBefforeMessage := True; // Importante!
  //FDefaultClientOriginPrepared := False;

  FFakeWindow := TPanel.Create(Self);
  with FFakeWindow do
  begin
    //ParentColor := False;
    //Color := clForm; colocado ao inicializar o handle!
    BorderWidth := 3;

    OnMouseDown := @FFakeWindowMouseDown;
    OnMouseMove := @FFakeWindowMouseMove;
    OnMouseUp   := @FFakeWindowMouseUp;
    OnResize    := @FFakeWindowResize;

  end;

  FFakeWindowCaption := TPanel.Create(Self);
  with FFakeWindowCaption do
  begin
    Parent := FFakeWindow;
    Align := alTop;
    Height := 18;
    BevelInner := bvNone;
    BevelOuter := bvNone;
    ParentColor := False;
    Color := clActiveCaption;
    OnPaint := @FFakeWindowCaptionPaint;
  end;

  Width := Screen.Width * 2;
  Height := Screen.Height * 2;

  Left := 0;
  Top  := 0;

  SetupScreen;

end;

destructor TDesignPanel.Destroy;
begin
  FreeAndNil(FFakeWindowCaption);
  FreeAndNil(FFakeWindow);
  inherited Destroy;
end;

procedure TDesignPanel.UnLoadLoadedForm;
begin

  if not LoadedFormIsValid then Exit;

  try
    FLoadingForm := True; // Important!
    UnLoadForm(LoadedForm);
    LoadedForm := Nil;
  finally
    FLoadingForm := False;
  end;

end;

procedure TDesignPanel.LoadLastForm;
begin

  if (FLastForm <> Nil) and (Assigned(FLastForm)) then
  begin
    //LoadForm(FLastForm);
    ShowWindow(FLastForm.Handle, SW_SHOW); // Assim funcioma melhor ;)
  end;

end;

procedure TDesignPanel.UpdateScreenForms;
var
  i: Integer;
begin

  // Esse procedimento é para sistemas em que o self
  // é preparado depois dos forms dsigners serem
  // adicionados à screen

  if not KZ_DOCK_FORM_DESIGN then Exit;

  for i := 0 to Screen.CustomFormCount -1 do
  begin
    if IsFormDesign(Screen.CustomForms[i]) then
      AddForm(Screen.CustomForms[i]);
  end;

end;

{ TTabComponentSearch }

procedure TTabComponentSearch.DestroyButtons;
begin

  while MyPanel.ControlCount > 0 do
  begin
    MyPanel.Controls[0].Free;
  end;

end;

function TTabComponentSearch.NewButton: TFindComponentButton;
begin
  Result := TFindComponentButton.Create(Self);
  Result.Parent := MyPanel;
end;

constructor TTabComponentSearch.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);

  MyLabel := TLabel.Create(Self);
  MyLabel.Parent := Self;
  MyLabel.Align := alTop;
  MyLabel.Font.Style := [fsBold];
  MyLabel.Caption := ' Search results';

  MyPanel := TPanel.Create(Self);
  MyPanel.Parent := Self;
  MyPanel.Align := alClient;
  MyPanel.BevelInner := bvNone;
  MyPanel.BevelOuter := bvNone;

  ShowHint := True;
  Align := alClient;
  Visible := False;
  BevelInner := bvNone;
  BevelOuter := bvNone;

end;

destructor TTabComponentSearch.Destroy;
begin
  DestroyButtons;

  FreeAndNil(MyLabel);
  FreeAndNil(MyPanel);

  inherited Destroy;

end;

{ TFindComponentButton }

procedure TFindComponentButton.Click;
var
  _OnChange: TNotifyEvent;
begin
  inherited Click;

  if Comp <> Nil then
  begin

    if (Parent <> nil) and (Parent.Parent <> nil) then
    with TTabComponentSearch(Parent.Parent) do
    begin
      Visible := False;
    end;

    ComponentReg.IDEComponentPalette.Selected := Comp;

    EXIT;

  end;


  if (Parent <> nil) and (Parent.Parent <> nil) then
    with TTabComponentSearch(Parent.Parent) do
    begin
      Visible := False;
    end;

  ComponentPage.Show;
  ComponentPage.SetFocus;

  with findEdt do
  begin
    try
      _OnChange := OnChange;
      OnChange := nil;
      Clear;
    finally
      OnChange := _OnChange;
    end;
  end;

  ComponentReg.IDEComponentPalette.Selected :=
    ComponentReg.IDEComponentPalette.FindComponent(FClassC);

end;

procedure TFindComponentButton.Prepare(_ComponentButton: TSpeedButton;
  _ComponentPage: TTabSheet; _findEdt: TCustomEdit; _Class: String);
begin

  ComponentButton := _ComponentButton;
  ComponentPage := _ComponentPage;
  findEdt := _findEdt;

  Glyph.Assign(ComponentButton.Glyph);
  Hint := ComponentButton.Hint;
  Width := ComponentButton.Width;
  ShowHint := True;
  Flat := True;
  GroupIndex := 1;
  Align := alLeft;
  FClassC := _Class;

end;

procedure TFindComponentButton.Prepare(_Comp: TRegisteredComponent);
begin

  Comp := _Comp;

  Glyph.Assign(TSpeedButton( Comp.Button ).Glyph);
  Hint := TSpeedButton( Comp.Button ).Hint;
  Width := TSpeedButton( Comp.Button ).Width;
  ShowHint := True;
  Flat := True;
  GroupIndex := 1;
  Align := alLeft;

end;

{$R *.lfm}

{ TKZLazDesktopInterface }

procedure TKZLazDesktopInterface.FormCreate(Sender: TObject);
begin

  pnKZTools.Visible := KZ_USE_KZTOOLS;

  {$IFDEF MODO_TESTE}

  Button1.Visible := True;
  Button2.Visible := True;

  {$ELSE}

  Button1.Visible := False;
  Button2.Visible := False;

  {$ENDIF}

  pcCorpo.ActivePageIndex := 0;

  with pnScreenPreview do
  begin
    Width := PixelPerc(Screen.Width, PreviewFactor);
    Height := PixelPerc(Screen.Height, PreviewFactor);
    DoubleBuffered := True;
  end;

  with pnScreenMove do
  begin
    Width := PixelPerc(Screen.Width, PreviewFactor);
    Height := PixelPerc(Screen.Height, PreviewFactor);
    DoubleBuffered := True;
  end;

  KZConfigManager := TKZConfigManager.Create;

  FDesignPanel := TDesignPanel.Create(Self);
  with FDesignPanel do
  begin
    Parent := pnDesign;
    Left := 0;
    Top := 0;
    Width := Screen.Width * 2;
    Height := Screen.Height * 2;
    BevelOuter := bvNone;
    OnFormLoad := @DesignPanel_OnLoadForm;
    OnLoadedFormChangeBounds := @DesignPanel_OnLoadedFormChangeBounds;
  end;

  pcInspectors.Width := PixelPerc(Screen.Width, 25);

  pcInspectors.ShowTabs := False;
  pcInspectors.ActivePageIndex := 0;

end;

procedure TKZLazDesktopInterface.FormShow(Sender: TObject);
begin

  // Form Show

  pcInspectors.Width := StrToInt( LoadCFG('Inspectors', 'Width', IntToStr(pcInspectors.Width)) );
  pnMessages.Height := StrToInt( LoadCFG('Messages', 'Height', IntToStr(pnMessages.Height)) );

end;

procedure TKZLazDesktopInterface.imgDesignAreaViewPaint(Sender: TObject);
var
  x, y: Integer;
  ARect: TRect;
begin

  with imgDesignAreaView do
  begin

    if
      Assigned( Canvas ) and
      (Self.Visible) and
      (Canvas.HandleAllocated)
    then
    begin
      {
      pnWindowScreenDesignAreaView.PaintTo(   // Isso seria o ideal, mas não funciona em todos os sistemas :(
        Canvas,
        (-Parent.Left) + 3,
        (-Parent.Top) + 3);
      }

      // Desenha o form
      x := (-Parent.Left) + 3;
      y := (-Parent.Top) + 3;

      Canvas.Brush.Color := pnWindowScreenDesignAreaView.Color;
      Canvas.Pen.Color := pnWindowScreenDesignAreaView.Color;
      Canvas.Rectangle(
        x,
        y,
        x + pnWindowScreenDesignAreaView.Width,
        y + pnWindowScreenDesignAreaView.Height);


      // Desenha o caption do form
      Canvas.Brush.Color := pnCaptionWindowScreenDesignAreaView.Color;
      Canvas.Pen.Color := pnCaptionWindowScreenDesignAreaView.Color;
      Canvas.Rectangle(
        x + pnCaptionWindowScreenDesignAreaView.Left,
        y + pnCaptionWindowScreenDesignAreaView.Top,
        x + pnCaptionWindowScreenDesignAreaView.Left + pnCaptionWindowScreenDesignAreaView.Width,
        y + pnCaptionWindowScreenDesignAreaView.Top + pnCaptionWindowScreenDesignAreaView.Height);


      // Desenha a borda do form
      ARect := Rect(x, y,
        x + pnWindowScreenDesignAreaView.Width,
        y + pnWindowScreenDesignAreaView.Height);

      Canvas.Frame3d(ARect,
        pnWindowScreenDesignAreaView.BevelWidth,
        pnWindowScreenDesignAreaView.BevelOuter);

      InflateRect(ARect,
        -pnWindowScreenDesignAreaView.BorderWidth,
        -pnWindowScreenDesignAreaView.BorderWidth);

      Canvas.Frame3d(ARect,
        pnWindowScreenDesignAreaView.BevelWidth,
        pnWindowScreenDesignAreaView.BevelInner);

      // Desenha a 'mira'
      Canvas.Pen.Color := clGreen;
      Canvas.Frame(
        Width div 4,
        Height div 4,
        Width - (Width div 4),
        Height - (Height div 4));

      Canvas.Pen.Color := clRed;
      Canvas.Frame(0, 0, Width, Height);
      Canvas.Line(0, Height div 2, Width, Height div 2);
      Canvas.Line(Width div 2, 0, Width div 2, Height);

    end;

  end;

end;

procedure TKZLazDesktopInterface.Button1Click(Sender: TObject);
var
  i: integer;
  p: TDesignPanel;
  pRect: TRect;
  bs: TFormBorderStyle;

  msg: TKZLMessageHandler;

  c: TKZConfigManager;
begin

  ObjectInspectorDlg.Visible := False;

end;

procedure TKZLazDesktopInterface.edFindComponentButtonClick(Sender: TObject);
begin
  edFindComponent.Clear;
end;

procedure TKZLazDesktopInterface.edFindComponentChange(Sender: TObject);
var
  iP, iC, iC2: integer;
  _page: TTabSheet;

  procedure _setuBtn(speed: TSpeedButton);
  begin

    with speed do
    begin

      if (Pos(LowerCase('PaletteSelectBtn'), LowerCase(Name)) = 0) and
        (Pos(LowerCase(edFindComponent.Text), LowerCase(Hint)) <> 0) then
      begin

        with TabComponentSearch.NewButton do
        begin
          Prepare(speed, _page, edFindComponent, 'TButton'); // bah
        end;

      end;

    end;
  end;

begin

  if TabComponentSearch = nil then
  begin
    TabComponentSearch := TTabComponentSearch.Create(Self);
    TabComponentSearch.Parent := pnComponents;
  end;

  with TabComponentSearch do
  begin
    DestroyButtons;
    Visible := False;
  end;

  if edFindComponent.Text = '' then
  begin
    TPageControl(pnComponents.FindChildControl('ComponentPageControl')).ActivePageIndex
    := 0;
    Exit;
  end;

  with TabComponentSearch do
  begin
    Visible := True;
  end;

  for iP := 0 to ComponentReg.IDEComponentPalette.Count - 1 do
  begin
    with ComponentReg.IDEComponentPalette.Pages[iP] do
    begin
      for iC := 0 to Count - 1 do
      begin
        if
          Pos(LowerCase(edFindComponent.Text),
          LowerCase( Items[iC].ComponentClass.ClassName )) <> 0
        then
        begin

          if (Items[iC].Button <> Nil) then
          begin

            Self.TabComponentSearch.NewButton.Prepare(Items[iC]);

          end;

        end;

      end;

    end;

  end;

end;

procedure TKZLazDesktopInterface.FormClose(Sender: TObject;
  var CloseAction: TCloseAction);
begin

  SaveCFG('Inspectors', 'Width', IntToStr( pcInspectors.Width ));
  SaveCFG('Messages', 'Height', IntToStr( pnMessages.Height ));

end;

procedure TKZLazDesktopInterface.imgWindowScreenPreviewMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin

  SetCapture(pnWindowScreenPreview.Handle);
  FPreviewMoving := True;
  FPreviewMovingPoint.X := x;
  FPreviewMovingPoint.Y := Y;

end;

procedure TKZLazDesktopInterface.imgWindowScreenPreviewMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: integer);
begin

  if FPreviewMoving then
  begin
    pnWindowScreenPreview.Left :=
      pnWindowScreenPreview.Left - (FPreviewMovingPoint.x - X);
    pnWindowScreenPreview.Top :=
      pnWindowScreenPreview.Top - (FPreviewMovingPoint.Y - Y);
  end;

end;

procedure TKZLazDesktopInterface.imgWindowScreenPreviewMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: integer);
begin

  if FPreviewMoving then
  begin
    ReleaseCapture;
    FPreviewMoving := False;

    with pnWindowScreenPreview do
    begin
      Left := max(Left - (FPreviewMovingPoint.X - X), 0);
      Top := max(Top - (FPreviewMovingPoint.Y - Y), 0);

      if FDesignPanel.LoadedFormIsValid then
      begin

        FDesignPanel.LoadedForm.SetBounds(
          round((Left / Parent.Width) * Screen.Width),
          round((Top / Parent.Height) * Screen.Height),
          FDesignPanel.LoadedForm.Width,
          FDesignPanel.LoadedForm.Height);

      end;

    end;
  end;

end;

procedure TKZLazDesktopInterface.MenuItem1Click(Sender: TObject);
var
  sPath: String;

  function _sep: String;
  begin
    {$IFDEF MSWINDOWS}
    Result := '\';
    {$ELSE}
    Result := '/';
    {$ENDIF}
  end;

begin


  try
    sPath := SourceEditorManagerIntf.ActiveSourceWindow.ActiveEditor.GetProjectFile.Filename;

    with MyProcess do
    begin
      CommandLine := '"' + sPath + _sep + 'android' + _sep + 'ant debug"';
      Execute;
    end;

    ShowMessage('Done!');

  except
    on Ex: Exception do
    begin
      ShowMessage('Error: ' + Ex.Message);
    end;
  end;

end;

procedure TKZLazDesktopInterface.PackageListClick(Sender: TObject);
begin

  if (PackageListHandles.Count = 0) or (PackageList.Items.Count = 0) or
    (PackageList.ItemIndex = -1) then
    Exit;

  TCustomForm(PackageListHandles.Items[PackageList.ItemIndex]).Show;

end;

procedure TKZLazDesktopInterface.pcCorpoChange(Sender: TObject);
begin

  // Aqui nós escondemos tudo o que vai dentro das abas. Isso porque
  // para que a aba seja chamada ao abrir a janela respectiva, a janela
  // deve estar invisível para executar a mensagem LM_SHOWWINDOW...

  if SourceEditorManager <> Nil then
  begin

    if pcCorpo.ActivePage <> tsCodeEditor then
      SourceEditorManager.HideForms
    else
    begin
      SourceEditorManager.ShowForms;


      if pcInspectors.ShowTabs then
        pcInspectors.ActivePage := tsCodeExplorer;


    end;

  end;

  if PackageManager <> Nil then
  begin
          {
    if pcCorpo.ActivePage <> tsPackage then
      PackageManager.HideForms
    else
      PackageManager.ShowForms;
          }
  end;

  if FDesignPanel <> Nil then
  begin

    if pcCorpo.ActivePage <> tsFormEditor then
      FDesignPanel.UnLoadLoadedForm
    else
    begin

      if not FDesignPanel.LoadedFormIsValid then
      begin
        FDesignPanel.LoadLastForm;
      end;

      if pcInspectors.ShowTabs then
        pcInspectors.ActivePage := tsObjectInspector;

    end;

  end;

end;

procedure TKZLazDesktopInterface.pnCodeEditorResize(Sender: TObject);
begin



end;

procedure TKZLazDesktopInterface.pnDesignAreaViewMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

  SetCapture(pnDesignAreaView.Handle);

  FDesignAreaViewMoving := True;
  FDesignAreaViewMovingPoint.X := x;
  FDesignAreaViewMovingPoint.Y := Y;

end;

procedure TKZLazDesktopInterface.pnDesignAreaViewMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin

  if FDesignAreaViewMoving then
  begin

    with pnDesignAreaView do
    begin
      Left := Left - (FDesignAreaViewMovingPoint.x - X);
      Top := Top - (FDesignAreaViewMovingPoint.Y - Y);
    end;

  end;

  imgDesignAreaView.Repaint;

end;

procedure TKZLazDesktopInterface.pnDesignAreaViewMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

  if FDesignAreaViewMoving then
  begin
    ReleaseCapture;
    FDesignAreaViewMoving := False;

    with pnDesignAreaView do
    begin

      Left := max(Left - (FDesignAreaViewMovingPoint.X - X), 0);
      Top := max(Top - (FDesignAreaViewMovingPoint.Y - Y), 0);

      if FDesignPanel.LoadedFormIsValid then
      begin

        FDesignPanel.SetBounds(
          (-FDesignPanel.LoadedForm.Left) - round(Left * (Screen.Width / Parent.Width)),
          (-FDesignPanel.LoadedForm.Top) - round(Top * (Screen.Height / Parent.Height)),
          FDesignPanel.Width,
          FDesignPanel.Height);

      end;


    end;

  end;

end;

procedure TKZLazDesktopInterface.pnPackageResize(Sender: TObject);
begin

end;

procedure TKZLazDesktopInterface.pnScreenMoveResize(Sender: TObject);
begin

  with pnScreenMove do
  begin
    SpeedButton1.Left := (Width div 2) - (SpeedButton1.Width div 2);
    SpeedButton1.Top := (Height div 2) - SpeedButton1.Height;
  end;

  with SpeedButton1 do
  begin
    SpeedButton2.Left := Left;
    SpeedButton2.Top := Top + Height;
  end;

  with SpeedButton2 do
  begin
    SpeedButton3.Left := Left - Width;
    SpeedButton3.Top := Top;

    SpeedButton4.Left := Left + Width;
    SpeedButton4.Top := Top;
  end;

end;

procedure TKZLazDesktopInterface.SpeedButton1Click(Sender: TObject);
begin

  with FDesignPanel do
  begin

    if Sender = SpeedButton1 then Top := Top - 25;
    if Sender = SpeedButton2 then Top := Top + 25;

    if Sender = SpeedButton3 then Left := Left - 25;
    if Sender = SpeedButton4 then Left := Left + 25;

  end;

end;

procedure TKZLazDesktopInterface.SpeedButton5Click(Sender: TObject);
begin
  ppmAndroid.PopUp(Mouse.CursorPos.x, Mouse.CursorPos.y);
end;

procedure TKZLazDesktopInterface.tsCodeEditorEnter(Sender: TObject);
begin

end;

procedure TKZLazDesktopInterface.tsFormEditorShow(Sender: TObject);
begin

  if Sender = tsPackage then
  begin

    pnPackageResize(pnPackage);

    UpdatePackageList;

  end;

end;

procedure TKZLazDesktopInterface.tsObjectInspectorResize(Sender: TObject);
begin

  {
  if
    (KZ_OBJECT_INSPECTOR_PARENT_TYPE = kzptParentWindow) and
    (ObjectInspectorDlg <> nil)
  then
  begin

    with ObjectInspectorDlg do
    begin
      Left := 0;
      Top  := 0;
      Width  := tsObjectInspector.Width;
      Height := tsObjectInspector.Height;
    end;

  end;
  }

end;

procedure TKZLazDesktopInterface.tsObjectInspectorShow(Sender: TObject);
begin

  if ObjectInspectorDlg <> nil then
    tsObjectInspector.Caption := ObjectInspectorDlg.Caption;

  if CodeExplorerView <> nil then
    tsCodeExplorer.Caption := CodeExplorerView.Caption;

end;

procedure TKZLazDesktopInterface.TESTE(Sender: TObject);
begin

  AddMessageDebug('Teste - Visivel');


end;

function TKZLazDesktopInterface.MainIDEForm: TCustomForm;
begin
  Result := Screen.FindForm('MainIDE');
end;

function TKZLazDesktopInterface.ObjectInspectorDlg: TObjectInspectorDlg;
var
  i: Integer;
begin

  //for i := 0 to Screen.CustomFormCount - 1 do
  //begin
  //  if Screen.CustomForms[i] is TObjectInspectorDlg then
  //  begin
  //    Result := TObjectInspectorDlg(Screen.CustomForms[i]);
  //    Break;
  //  end;
  //end;

  Result := TObjectInspectorDlg(IDEWindowCreators.GetForm(
    DefaultObjectInspectorName, True));

end;

function TKZLazDesktopInterface.MessagesView: TIDEMessagesWindowInterface;
begin
  Result := TIDEMessagesWindowInterface(IDEWindowCreators.GetForm('MessagesView', True));
end;

function TKZLazDesktopInterface.CodeExplorerView: TCustomForm;
begin
  Result := IDEWindowCreators.GetForm('CodeExplorerView', True);
end;

function TKZLazDesktopInterface.pnlSpeedButtons: TPanel;
begin
  Result := TPanel(MainIDE.OwningComponent.FindComponent('pnlSpeedButtons'));
end;

function TKZLazDesktopInterface.ComponentPageControl: TPageControl;
begin
  Result := TPageControl(MainIDE.OwningComponent.FindComponent('ComponentPageControl'));
end;

procedure TKZLazDesktopInterface.Screen_FormAdded(Sender: TObject; Form: TCustomForm);
begin

  if (pcCorpo = nil) or (Form = nil) then
    Exit;

  // Packages
  if (KZ_DOCK_PACKAGE) and (IsPackage(Form)) then SetupPackage(Form);

  // Sources Editors
  if IsSourceEditor(Form) then SetupSourceEditor(Form);

  //if IsCodeExplorerView(Form) then SetupCodeExplorerView;

end;


procedure TKZLazDesktopInterface.SetupScreen;
begin

  Screen.AddHandlerFormAdded(@Screen_FormAdded, False);
  //Screen.AddHandlerActiveFormChanged(@Screen_ActiveFormChanged, False);
  //Screen.AddHandlerActiveControlChanged(@Screen_ActiveControlChanged, False);

end;

procedure TKZLazDesktopInterface.MainIDEFormShow(TheOwner: TObject);
begin

  SetupDefaultBars;

end;

procedure TKZLazDesktopInterface.OnIDERestoreWindows(Sender: TObject);
begin

  //SetupSourceNotebook; lazarus works with a varios sources notebooks...
  //                     See the Screen_FormAdded Screen_FormAdded
  //SetupMessages;
  //SetupCodeExplorerView;
  //SetupObjectInspector;

  // Começa agora o novo esquema de controle ;)
  ObjectInspectorManager.Prepare(ObjectInspectorDlg);
  CodeExplorerViewManager.Prepare(CodeExplorerView);
  MessagesViewManager.Prepare(MessagesView);

  UpdateFromOptions;

  if FDesignPanel <> Nil then
    FDesignPanel.UpdateScreenForms;

  //if IDEMessagesWindow <> Nil then
  //begin
  //
  //  with IDEMessagesWindow do
  //  begin
  //    AddMsg('Welcome to KZDesktop! - www.raphaelz.com.br', '', 1);
  //    AddMsg('Remember: This is a beta version. Bugs? Ignore, please! ;)', '', 2);
  //
  //  end;
  //
  //end;

end;

procedure TKZLazDesktopInterface.MainIDEFormResize(Sender: TObject);
begin

  with TWinControl(Sender).ClientRect do
  begin
    Self.Left   := Left;
    Self.Top    := Top;
    Self.Width  := Right - Left;
    Self.Height := Bottom - Top;
  end;

end;

function TKZLazDesktopInterface.OnProjectClose(Sender: TObject;
  AProject: TLazProject): TModalResult;
begin
  pcCorpo.ActivePage := tsCodeEditor; // Importante para evitar paus
end;

procedure TKZLazDesktopInterface.BefforeMessageWindow(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);
begin


  {
  if IsSourceEditor( TCustomForm( SenderWindow ) ) then
  begin

    // Se está exibindo a tela...
    if Msg.msg in [LM_SHOWWINDOW] then
    begin

      //AddMessageDebug('SourceEditor (beffore) - VISIBLE ' + BoolToStr(SenderWindow.Visible, True));

      if TLMShowWindow(Msg).Show then
      begin

        pcCorpo.ActivePage := tsCodeEditor;

      end;

    end;

  end;

  if IsCodeExplorerView( TCustomForm( SenderWindow ) ) then
  begin

    // Se está exibindo a tela...
    if Msg.msg in [LM_SHOWWINDOW] then
    begin

      if TLMShowWindow(Msg).Show then
      begin
        pcInspectors.ShowTabs := True;
        pcInspectors.ActivePage := tsCodeExplorer;
      end;

    end;

  end;
  }

end;

procedure TKZLazDesktopInterface.AfterMessageWindow(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);

{
  procedure SetFocusInSourceEditor;
  begin

    with TSourceEditorWindowInterface(SenderWindow) do
    begin
      if (ActiveEditor <> Nil) then
      begin
        //AddMessageDebug('ActiveEditor <> Nil');

        if (ActiveEditor.EditorControl <> Nil) then
        begin

          //AddMessageDebug('ActiveEditor.EditorControl.CanFocus = ' +
            //BoolToStr(ActiveEditor.EditorControl.CanFocus, True));

          if (ActiveEditor.EditorControl.CanFocus) then
            if not ActiveEditor.EditorControl.Focused then
              ActiveEditor.EditorControl.SetFocus;

        end;

      end
      else
        //AddMessageDebug('ActiveEditor = Nil');

    end;
  end;
  }

begin


  {

  if Pos('Unknown message', GetMessageName(Msg.msg)) = 0 then
  AddMessageDebug('SourceEditor (After) - ' + GetMessageName(Msg.msg));

  if bahhfing then Exit; // teste


  if IsSourceEditor( TCustomForm( SenderWindow ) ) then
  begin

    // Se não está trabalhando em modo ParentWindow, esse esquema não se aplica...
    if ParentWindow = 0 then Exit;

    //if Pos('Unknown message', GetMessageName(Msg.msg)) = 0 then
      AddMessageDebug('SourceEditor (After) - ' + GetMessageName(Msg.msg));

    // Se está exibindo a tela...
    if Msg.msg in [LM_SHOWWINDOW {, LM_ACTIVATE, LM_SETFOCUS}] then
    begin

      if TLMShowWindow(Msg).Show then
      begin

        pcCorpo.ActivePage := tsCodeEditor;

        //SetFocusInSourceEditor;

      end;

    end;

    // Se está entrando em edição...
    //if Msg.msg in [LM_SETEDITABLE] then
    if Msg.msg = LM_SETEDITABLE then // Essa mensagem não pode ser usada em um 'IN' pois
    begin                            // ela é uma soma de outras mensagens!
      //AddMessageDebug('SetFocusInSourceEditor - Starting');
      //SetFocusInSourceEditor;

    end;

  end;
  }

end;

procedure TKZLazDesktopInterface.AfterMessageEditor(Sender: TObject;
  SenderWindow: TControl; Msg: TLMessage);
begin

end;

procedure TKZLazDesktopInterface.DesignPanel_OnLoadForm(Sender: TObject);
begin
  pcCorpo.ActivePage := tsFormEditor;
end;

procedure TKZLazDesktopInterface.DesignPanel_OnLoadedFormChangeBounds(
  Sender: TObject);
begin

  if FDesignPanel.LoadedFormIsValid then
  begin
    SetupPreviewFormDesign(FDesignPanel.LoadedForm);
    SetupFormDesignAreaView(FDesignPanel.LoadedForm);
  end;

end;

function TKZLazDesktopInterface.IsSourceEditor(fm: TCustomForm): boolean;
begin
  //Result := (fm = SourceNotebook) and (not (csDesigning in fm.ComponentState));
  Result := (fm is TSourceEditorWindowInterface);
end;

function TKZLazDesktopInterface.IsObjectInspector(fm: TCustomForm): boolean;
begin
  Result := (fm = ObjectInspectorDlg) and (not (csDesignInstance in fm.ComponentState));
end;

function TKZLazDesktopInterface.IsMessages(fm: TCustomForm): boolean;
begin
  Result := (fm = MessagesView) and (not (csDesignInstance in fm.ComponentState));
end;

function TKZLazDesktopInterface.IsFormDesign(fm: TCustomForm): boolean;
begin
  Result := ((csDesignInstance in fm.ComponentState) or
    (fm.ClassName = 'TNonControlDesignerForm') or (fm.ClassName =
    'TFrameDesignerForm'));
end;

function TKZLazDesktopInterface.IsPackage(fm: TCustomForm): boolean;
begin
  Result := (fm.ClassName = 'TPackageEditorForm');
end;

function TKZLazDesktopInterface.IsCodeExplorerView(fm: TCustomForm): boolean;
begin
  Result := (fm.ClassName = 'TCodeExplorerView');
end;

function TKZLazDesktopInterface.PixelPerc(vl, perc: single): integer;
begin
  Result := Round((vl / 100) * perc);
end;

procedure TKZLazDesktopInterface.SetupPreviewFormDesign(_Form: TCustomForm);
begin

  if (pcCorpo.ActivePage <> tsFormEditor) or (FPreviewMoving) then
    Exit;

  with pnWindowScreenPreview do
  begin
    Width := PixelPerc(_Form.Width, PreviewFactor);
    Height := PixelPerc(_Form.Height, PreviewFactor);
    Top := PixelPerc(_Form.Top, PreviewFactor);
    Left := PixelPerc(_Form.Left, PreviewFactor);
    DoubleBuffered := True;
  end;

  with pnCaptionWindowScreenPreview do
  begin
    Height := PixelPerc(Parent.Height, 15);
  end;

end;

procedure TKZLazDesktopInterface.SetupFormDesignAreaView(_Form: TCustomForm);
begin

  if
    (pcCorpo.ActivePage <> tsFormEditor)
  then
    Exit;

  with pnWindowScreenDesignAreaView do
  begin
    Width := PixelPerc(_Form.Width, PreviewFactor);
    Height := PixelPerc(_Form.Height, PreviewFactor);
    Top := 3;
    Left := 3;
    DoubleBuffered := True;
  end;

  with pnCaptionWindowScreenDesignAreaView do
  begin
    Height := PixelPerc(Parent.Height, 15);
  end;

  with pnDesignAreaView do
  begin
    Left := 0;
    Top := 0;
    Width  := PixelPerc(pnDesign.Width, PreviewFactor);
    Height := PixelPerc(pnDesign.Height, PreviewFactor);
  end;

end;

procedure TKZLazDesktopInterface.SetupDefaultBars;
var
  aToolBarDeb, aToolBarStd: TToolbar; //SunyD
  i, w: integer; //Sunyd
begin

  with TKZLazDesktopInterface(Self) do
  begin

    if (pnlSpeedButtons = nil) then
      Exit;

    with pnlSpeedButtons do
    begin
      //Suny: Added from me
      aToolBarStd := TToolBar(FindChildControl('tbStandard'));
      aToolBarStd.EdgeBorders := [];
      aToolBarStd.Wrapable := False;
      aToolBarStd.Align := alNone;
      w := 0;
      for i := 0 to aToolBarStd.ControlCount - 1 do
        w := w + aToolBarStd.Controls[i].Width;
      aToolBarStd.SetBounds(BorderWidth, BorderWidth, w + 2, aToolBarStd.ButtonHeight);

      aToolBarDeb := TToolBar(FindChildControl('tbViewDebug'));
      aToolBarDeb.EdgeBorders := [];
      aToolBarDeb.Wrapable := False;
      aToolBarDeb.Align := alNone;
      w := 0;
      for i := 0 to aToolBarDeb.ControlCount - 1 do
        w := w + aToolBarDeb.Controls[i].Width;
      aToolBarDeb.SetBounds(aToolBarStd.Left + aToolBarStd.Width,
        BorderWidth, w + 2, aToolBarDeb.ButtonHeight);
      //End ... Suny: Added from me

      {Code from Raphael
      TToolBar(FindChildControl('tbViewDebug')).Align := alLeft;
      TToolBar(FindChildControl('tbStandard')).Align := alLeft;
      }

      pnMenu.Height := aToolBarStd.Height + 4;   //Added from Suny;

      Align := alClient;
      Parent := pnMenu;
    end;

    if ComponentPageControl <> nil then
    begin

      {$IFDEF MSWINDOWS}
      pnComponents.Height := GetSystemMetrics(SM_CYCAPTION) + 35;
      {$ELSE}
      pnComponents.Height := 75;
      {$ENDIF}

      ComponentPageControl.Parent := pnComponents;

    end;

  end;

end;

{
procedure TKZLazDesktopInterface.SetupCodeExplorerView;
begin

  CodeExplorerViewManager.Prepare(CodeExplorerView, tsCodeExplorer, kzptParentWindow);

  EXIT; // Teste

  if (CodeExplorerView <> nil) then
  begin

    if CodeExplorerView.Parent <> tsCodeExplorer then
    begin
      with CodeExplorerView do
      begin
        BorderStyle := bsNone;
        Parent := tsCodeExplorer;
        Align := alClient;
      end;

      tsObjectInspectorShow(tsCodeExplorer);

    end;

    with TKZLMessageHandler.Create(CodeExplorerView) do
    begin
      OnBefforeMessage := @Self.BefforeMessageWindow;
      OnAfterMessage   := @Self.AfterMessageWindow;
    end;

    //tsCodeExplorer.TabVisible := True;
    //pcInspectors.ShowTabs := CodeExplorerView.Visible;

  end
  else
  begin
    //tsCodeExplorer.TabVisible := False;
    //pcInspectors.ShowTabs := False;
  end;

  //if not pcInspectors.ShowTabs then
  //  pcInspectors.ActivePageIndex := 0;

end;
}

{
procedure TKZLazDesktopInterface.SetupObjectInspector;
var
  aRect: TRect;
begin

  ObjectInspectorManager.Prepare(ObjectInspectorDlg, tsObjectInspector, KZ_OBJECT_INSPECTOR_PARENT_TYPE);

  EXIT;

  with ObjectInspectorDlg do
  begin
    BorderStyle := bsNone;

    aRect.Left := 0;
    aRect.Top  := 0;
    aRect.Right := Width;
    aRect.Bottom := Height;

    case KZ_OBJECT_INSPECTOR_PARENT_TYPE of
      kzptParent       :
      begin
        Parent := tsObjectInspector;
        Align := alClient;
      end;

      kzptParentWindow :
      begin
        ParentWindow := tsObjectInspector.Handle;
        tsObjectInspectorResize(tsObjectInspector);
      end;
      kzptDock         :
      begin
        Dock(tsObjectInspector, aRect);
        Align := alClient;
      end;

      kzptNone         : Parent := Nil;

    end;

  end;

  ObjectInspectorDlg.FindChildControl('StatusBar').Visible := False;

end;
}

{
procedure TKZLazDesktopInterface.SetupMessages;
begin

  MessagesViewManager.Prepare(MessagesView, pnMessages, kzptParent);

  EXIT;

  pnMessages.BevelOuter := bvNone;

  with MessagesView do
  begin
    BorderStyle := bsNone;
    Parent := pnMessages;
    Align := alClient;
  end;

  if MessagesView <> Nil then
  begin

    with MessagesView do
    begin
      AddMsg('Welcome to KZDesktop! - www.raphaelz.com.br', '', 1);
      AddMsg('Remember: This is a beta version. Bugs? Ignore, please! ;)', '', 2);

    end;

  end;

end;
}

procedure TKZLazDesktopInterface.SetupPackage(pkg: TCustomForm);
begin

  if not KZ_DOCK_PACKAGE then Exit;

  PackageManager.AddForm(pkg);

  pcCorpo.ActivePage := tsPackage;

  //UpdatePackageList(pkg);




  EXIT; // TESTE

  try
    pnPackage.Visible := False; // Esconde, pois se estiver visível, dá pau na
    // hora de carregar o pacote!

    FEnterInEditor := False;

    with pkg do
    begin
      BorderStyle := bsNone;

      case KZ_PACKAGE_PARENT_TYPE of
        kzptParent       : Parent := pnPackage;
        kzptParentWindow : ParentWindow := pnPackage.Handle;
        kzptNone         : Parent := Nil;
      end;

      Align := alClient;

    end;

    try
      tsPackage.OnShow := nil; // Para não executar duas vezes
      pcCorpo.ActivePage := tsPackage;
      tsFormEditorShow(tsPackage);

    finally
      tsPackage.OnShow := @tsFormEditorShow;
    end;

  finally
    pnPackage.Visible := True;
    FEnterInEditor := True;
  end;

  UpdatePackageList;

end;

procedure TKZLazDesktopInterface.SetupSourceEditor(_Editor: TCustomForm);
begin

  if not KZ_DOCK_SOURCE_EDITOR then Exit;

  SourceEditorManager.AddForm(_Editor);

end;

procedure TKZLazDesktopInterface.UpdatePackageList(AddForm: TCustomForm);
var
  i: integer;
  sSelected: string;
begin

  if PackageList.ItemIndex > -1 then
    sSelected := PackageList.Items[PackageList.ItemIndex];

  PackageList.Items.Clear;

  if PackageListHandles = nil then
    PackageListHandles := TList.Create;

  PackageListHandles.Clear;

  for i := 0 to Screen.CustomFormCount - 1 do
  begin
    if IsPackage(Screen.CustomForms[i]) and (Screen.CustomForms[i] <> AddForm) then
    begin
      with Screen.CustomForms[i] do
      begin
        Self.PackageList.Items.Add(Caption);
        PackageListHandles.Add(Screen.CustomForms[i]);
      end;
    end;
  end;

  if AddForm <> Nil then
  begin
    with AddForm do
    begin
      Self.PackageList.Items.Add(Caption);
      PackageListHandles.Add(AddForm);
    end;

  end;

  if (sSelected <> '') and (PackageList.Items.Count > 0) then
  begin
    PackageList.ItemIndex := PackageList.Items.IndexOf(sSelected);

    if PackageList.ItemIndex > -1 then
      PackageListClick(PackageList);

  end;

end;

function TKZLazDesktopInterface.MainIDE: TLazIDEInterface;
var
  i: integer;
begin

  for i := 0 to Application.ComponentCount - 1 do
  begin
    if Application.Components[i].ClassName = 'TMainIDE' then
    begin
      Result := TLazIDEInterface(Application.Components[i]);
      Break;
    end;
  end;

end;

procedure TKZLazDesktopInterface.Execute;
begin

  Self.Left := 0;
  Self.Top := 0;

  //Self.Parent := MainIDEForm;
  //Self.Align := alClient;

  // Para nossa janela principal, usar o ParentWindow é melhor. Isso porque as
  // mensagens de teclas (por exemplo, CTRL+V no ObjectInspector), passam a funcionar
  // corretamente - pois a janela é tratada como janela, e não como componente...
  {$IFDEF WINDOWS}
  Self.ParentWindow := MainIDEForm.Handle;
  {$ELSE}
  Self.Parent := MainIDEForm; // No linux o ParentWindow não dá muito certo...
  {$ENDIF}

  MainIDEForm.AddHandlerOnResize(@MainIDEFormResize);

  Self.Visible := True;

  MainIDEForm.Constraints.MaxHeight := 0;
  MainIDEForm.WindowState := wsMaximized;
  MainIDEForm.AddHandlerFirstShow(@MainIDEFormShow, False);
  MainIDEFormResize(MainIDEForm);

  CodeExplorerViewManager   := TKZCodeExplorerViewManager.Create(tsCodeExplorer, kzptParentWindow);
  ObjectInspectorManager    := TKZObjectInspectorManager.Create(tsObjectInspector, KZ_OBJECT_INSPECTOR_PARENT_TYPE);
  MessagesViewManager       := TKZMessagesViewManager.Create(pnMessages, kzptParent);
  PackageManager            := TKZPackageManager.Create(pnPackage, KZ_PACKAGE_PARENT_TYPE);
  SourceEditorManager       := TKZSourceEditorManager.Create(pnCodeEditor, KZ_SOURCE_EDITOR_PARENT_TYPE);

  LazarusIDE.AddHandlerOnIDERestoreWindows(@OnIDERestoreWindows);
  LazarusIDE.AddHandlerOnProjectClose(@OnProjectClose);

  //SourceEditorManagerIntf.RegisterChangeEvent(semEditorActivate, @SourceEditorManager_EditorActivate);

  SetupScreen;

end;

procedure TKZLazDesktopInterface.SaveLog(sLog: string);
var
  log: TStringList;
  f: string;
  i: integer;
begin

  try
    log := TStringList.Create;

    f := ExtractFilePath(Application.ExeName) + 'kzdesktop.log';

    if FileExists(f) then
      log.LoadFromFile(f);

    if log.Count > 100 then
    begin
      for i := 1 to 70 do
      begin
        log.Delete(0);
      end;

    end;

    log.Add(FormatDateTime('YYYY-MM-DD hh:mm:ss', Now) + ' -> ' + sLog);

    log.SaveToFile(f);

  finally
    FreeAndNil(log);
  end;

end;

procedure TKZLazDesktopInterface.UpdateFromOptions;
begin

  if KZConfigManager.ObjectInspector_Align = KZCONF_ALIGN_LEFT then
  begin
    pcInspectors.Align := alLeft;
    spObjectInspector.Align := alLeft;
    spObjectInspector.Left := pcInspectors.Left + pcInspectors.Width + 2;
    pnFindComponent.Align := alLeft;
  end
  else
  begin
    pcInspectors.Align := alRight;
    spObjectInspector.Align := alRight;
    spObjectInspector.Left := pcInspectors.Left - pcInspectors.Width - 2;
    pnFindComponent.Align := alRight;
  end;

  if KZConfigManager.MessagesView_Location = KZCONF_LOCATION_IDE_BOTTON then
  begin
    pnMessages.Parent := pnCorpo;
    Splitter1.Parent := pnCorpo;
  end
  else
  begin
    pnMessages.Parent := tsCodeEditor;
    Splitter1.Parent := tsCodeEditor;
  end;

  pnDesign.Color := StringToColor(KZConfigManager.DesignEditor_BkCollor);

end;

end.

