unit uKZConfig;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  ButtonPanel, StdCtrls, ColorBox, Buttons,

  IDEOptionsIntf, typinfo, variants;

const
  KZCONF_LOCATION_TAB              = 'tab';
  KZCONF_LOCATION_SOURCE_EDITOR    = 'source_editor';
  KZCONF_LOCATION_IDE_BOTTON       = 'ide_bottom';
  KZCONF_LOCATION_OBJECT_INSPECTOR = 'object_inspector';
  KZCONF_ALIGN_LEFT                = 'left';
  KZCONF_ALIGN_RIGHT               = 'right';

type

  { TKZConfigManager }

  TKZConfigManager = class(TObject)
  private

    //FKzActive: Boolean;

    FCodeExplorer_Location: String;
    FDesignEditor_BkCollor: String;
    FMessagesView_Location: String;
    FObjectInspector_Align: String;

  published

    //property KzActive: Boolean read FKzActive write FKzActive;

    // Ohoh, as strings ficaram do mesmo tamanho ;) rs
    property ObjectInspector_Align: String read FObjectInspector_Align write FObjectInspector_Align;
    property CodeExplorer_Location: String read FCodeExplorer_Location write FCodeExplorer_Location;
    property MessagesView_Location: String read FMessagesView_Location write FMessagesView_Location;
    property DesignEditor_BkCollor: String read FDesignEditor_BkCollor write FDesignEditor_BkCollor;

  public

    procedure LoadDefaults;

    procedure Load;
    procedure Save;

    constructor Create;

  end;

  { TfmKZConfig }

  TfmKZConfig = class(TAbstractIDEOptionsEditor)
    ColorBox1: TColorBox;
    GroupBox1: TGroupBox;
    rgMessageViewLocation: TRadioGroup;
    rgCodeExplorerLocation: TRadioGroup;
    rgObjectInspectorPanel: TRadioGroup;
  private
    { private declarations }

  public
    { public declarations }

    constructor Create(AOwner: TComponent); override;
    function GetTitle: String; override;
    procedure Setup(ADialog: TAbstractOptionsEditorDialog); override;
    procedure ReadSettings(AOptions: TAbstractIDEOptions); override;
    procedure WriteSettings(AOptions: TAbstractIDEOptions); override;
    class function SupportedOptionsClass: TAbstractIDEOptionsClass; override;

  end; 

var
  fmKZConfig: TfmKZConfig;
  KZConfigManager: TKZConfigManager;

implementation

uses
  kzdesktop, KZLazDesktop;

{ TKZConfigManager }

constructor TKZConfigManager.Create;
begin
  inherited Create;

  Load;

end;

procedure TKZConfigManager.LoadDefaults;
begin

  //FKzActive := False;

  FCodeExplorer_Location := KZCONF_LOCATION_TAB;
  FDesignEditor_BkCollor := ColorToString(clGray);
  FMessagesView_Location := KZCONF_LOCATION_IDE_BOTTON;
  FObjectInspector_Align := KZCONF_ALIGN_LEFT;

end;

procedure TKZConfigManager.Load;
var
  Count, Size, I: Integer;
  List: PPropList;
  PropInfo: PPropInfo;
begin

  LoadDefaults; // Primeiro, prepara os padrões

  Count := GetPropList(Self.ClassInfo, tkAny, nil);
  Size  := Count * SizeOf(Pointer);
  GetMem(List, Size);

  try
    Count := GetPropList(Self.ClassInfo, tkProperties, List);

    for I := 0 to Count - 1 do
    begin

      PropInfo := List^[I];

      SetPropValue(Self, PropInfo^.Name,
        LoadCFG('Layout', PropInfo^.Name, VarToStr(GetPropValue(Self, PropInfo^.Name))));

    end;
  finally
    FreeMem(List);
  end;

end;

procedure TKZConfigManager.Save;
var
  Count, Size, I: Integer;
  List: PPropList;
  PropInfo: PPropInfo;
begin

  Count := GetPropList(Self.ClassInfo, tkAny, nil);
  Size  := Count * SizeOf(Pointer);
  GetMem(List, Size);

  try
    Count := GetPropList(Self.ClassInfo, tkProperties, List);

    for I := 0 to Count - 1 do
    begin

      PropInfo := List^[I];

      SaveCFG('Layout', PropInfo^.Name, VarToStr(GetPropValue(Self, PropInfo^.Name)));

    end;
  finally
    FreeMem(List);
  end;

end;

{ TfmKZConfig }

constructor TfmKZConfig.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

function TfmKZConfig.GetTitle: String;
begin
  Result := 'KZDesktop IDE';

end;

procedure TfmKZConfig.Setup(ADialog: TAbstractOptionsEditorDialog);
begin
  // Nada
end;

procedure TfmKZConfig.ReadSettings(AOptions: TAbstractIDEOptions);
begin

  with KZConfigManager do
  begin
    Load;

    if ObjectInspector_Align = KZCONF_ALIGN_LEFT then
      rgObjectInspectorPanel.ItemIndex := 0
    else
      rgObjectInspectorPanel.ItemIndex := 1;

    if CodeExplorer_Location = KZCONF_LOCATION_SOURCE_EDITOR then
      rgCodeExplorerLocation.ItemIndex := 0
    else
      rgCodeExplorerLocation.ItemIndex := 1;

    if MessagesView_Location = KZCONF_LOCATION_IDE_BOTTON then
      rgMessageViewLocation.ItemIndex := 0
    else
      rgMessageViewLocation.ItemIndex := 1;

    ColorBox1.Selected := StringToColor(DesignEditor_BkCollor);

  end;


  EXIT; // Novo acima

  if LoadCFG('Layout', 'ObjectInspectorPanel', 'right') = 'left' then
    rgObjectInspectorPanel.ItemIndex := 0
  else
    rgObjectInspectorPanel.ItemIndex := 1;


  if LoadCFG('Layout', 'CodeExplorerLocation', 'code_editor') = 'code_editor' then
    rgCodeExplorerLocation.ItemIndex := 0
  else
    rgCodeExplorerLocation.ItemIndex := 1;

  if LoadCFG('Layout', 'MessageViewLocation', 'botton_ide') = 'botton_ide' then
    rgMessageViewLocation.ItemIndex := 0
  else
    rgMessageViewLocation.ItemIndex := 1;

  ColorBox1.Selected := StringToColor(LoadCFG('Design Editor', 'background_color', ColorToString(clGray)));

end;

procedure TfmKZConfig.WriteSettings(AOptions: TAbstractIDEOptions);
begin

  with KZConfigManager do
  begin

    if rgObjectInspectorPanel.ItemIndex = 0 then
      ObjectInspector_Align := KZCONF_ALIGN_LEFT
    else
      ObjectInspector_Align := KZCONF_ALIGN_RIGHT;

    if rgCodeExplorerLocation.ItemIndex = 0 then
      CodeExplorer_Location := KZCONF_LOCATION_SOURCE_EDITOR
    else
      CodeExplorer_Location := KZCONF_LOCATION_OBJECT_INSPECTOR;

    if rgMessageViewLocation.ItemIndex = 0 then
      MessagesView_Location := KZCONF_LOCATION_IDE_BOTTON
    else
      MessagesView_Location := KZCONF_LOCATION_SOURCE_EDITOR;

    DesignEditor_BkCollor   := ColorToString(ColorBox1.Selected);

    Save;

  end;

  if Assigned(KZLazDesktopInterface) then
    KZLazDesktopInterface.UpdateFromOptions;


  EXIT; // Novo acima


  if rgObjectInspectorPanel.ItemIndex = 0 then
    SaveCFG('Layout', 'ObjectInspectorPanel', 'left')
  else
    SaveCFG('Layout', 'ObjectInspectorPanel', 'right');

  if rgCodeExplorerLocation.ItemIndex = 0 then
    SaveCFG('Layout', 'CodeExplorerLocation', 'code_editor')
  else
    SaveCFG('Layout', 'CodeExplorerLocation', 'object_inspector');

  if rgMessageViewLocation.ItemIndex = 0 then
    SaveCFG('Layout', 'MessageViewLocation', 'botton_ide')
  else
    SaveCFG('Layout', 'MessageViewLocation', 'botton_code_editor');


  SaveCFG('Design Editor', 'background_color', ColorToString(ColorBox1.Selected));


  if Assigned(KZLazDesktopInterface) then
    KZLazDesktopInterface.UpdateFromOptions;

end;

class function TfmKZConfig.SupportedOptionsClass: TAbstractIDEOptionsClass;
begin
  Result := TAbstractIDEEnvironmentOptions;
end;

{$R *.lfm}

end.

